@@ -30,15 +30,102 @@ using System.IO;
 using System.Collections.Generic;
 
 namespace Mono.Profiler {
+	public enum MonitorEvent {
+		CONTENTION = 1,
+		DONE = 2,
+		FAIL = 3
+	}
+	
+	public class LoadedAssembly {
+		uint id;
+		public uint ID {
+			get {
+				return id;
+			}
+		}
+		
+		string name;
+		public string Name {
+			get {
+				return name;
+			}
+		}
+		string baseName;
+		public string BaseName {
+			get {
+				return baseName;
+			}
+		}
+		uint major;
+		public uint Major {
+			get {
+				return major;
+			}
+		}
+		uint minor;
+		public uint Minor {
+			get {
+				return minor;
+			}
+		}
+		uint build;
+		public uint Build {
+			get {
+				return build;
+			}
+		}
+		uint revision;
+		public uint Revision {
+			get {
+				return revision;
+			}
+		}
+		string culture;
+		public string Culture {
+			get {
+				return culture;
+			}
+		}
+		string publicKeyToken;
+		public string PublicKeyToken {
+			get {
+				return publicKeyToken;
+			}
+		}
+		bool retargetable;
+		public bool Retargetable {
+			get {
+				return retargetable;
+			}
+		}
+		
+		public static readonly LoadedAssembly Unavailable = new LoadedAssembly (0, "UNAVAILABLE", "UNAVAILABLE", 0, 0, 0, 0, "neutral", "null", false);
+		
+		public LoadedAssembly (uint id, string name, string baseName, uint major, uint minor, uint build, uint revision, string culture, string publicKeyToken, bool retargetable) {
+			this.id = id;
+			this.name = name;
+			this.baseName = baseName;
+			this.major = major;
+			this.minor = minor;
+			this.build = build;
+			this.revision = revision;
+			this.culture = culture;
+			this.publicKeyToken = publicKeyToken;
+			this.retargetable = retargetable;
+		}
+	}
+	
 	public interface ILoadedElement {
 		uint ID {get;}
 		string Name {get;}
 	}
 	public interface ILoadedClass : ILoadedElement {
 		uint Size {get;}
+		LoadedAssembly Assembly {get;}
 	}
 	public interface ILoadedMethod<LC> : ILoadedElement where LC : ILoadedClass {
 		LC Class {get;}
+		bool IsWrapper {get;}
 	}
 	public interface IUnmanagedFunctionFromID<MR,UFR> : ILoadedElement where UFR : IUnmanagedFunctionFromRegion<UFR> where MR : IExecutableMemoryRegion<UFR> {
 		MR Region {get;}
@@ -102,8 +189,9 @@ namespace Mono.Profiler {
 	}
 	
 	public interface ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
-		LC NewClass (uint id, string name, uint size);
-		LM NewMethod (uint id, LC c, string name);
+		LoadedAssembly NewAssembly (uint id, string name, string baseName, uint major, uint minor, uint build, uint revision, string culture, string publicKeyToken, bool retargetable);
+		LC NewClass (uint id, LoadedAssembly assembly, string name, uint size);
+		LM NewMethod (uint id, LC c, bool isWrapper, string name);
 		MR NewExecutableMemoryRegion (uint id, string fileName, uint fileOffset, ulong startAddress, ulong endAddress);
 		UFI NewUnmanagedFunction (uint id, string name, MR region);
 		HS NewHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, TimeSpan headerStartTime, LC[] initialAllocations, bool recordSnapshot);
@@ -111,6 +199,8 @@ namespace Mono.Profiler {
 	}
 	
 	public interface ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+		LoadedAssembly[] Assemblies {get;}
+		LoadedAssembly GetAssembly (uint id);
 		LC[] Classes {get;}
 		LC GetClass (uint id);
 		LM[] Methods {get;}
@@ -131,18 +221,36 @@ namespace Mono.Profiler {
 		Double TicksPerCounterUnit {get;}
 		TimeSpan ClicksToTimeSpan (ulong clicks);
 		
+		/* Properties that state what kind of data has been processed */
+		bool HasAllocationData {get;}
+		void AllocationDataProcessed ();
+		bool HasCallData {get;}
+		void CallDataProcessed ();
+		bool HasStatisticalData {get;}
+		void StatisticalDataProcessed ();
+		bool HasHeapSnapshotData {get;}
+		void HeapSnapshotDataProcessed ();
+		bool HasJitTimeData {get;}
+		void JitTimeDataProcessed ();
+		bool HasGcTimeData {get;}
+		void GcTimeDataProcessed ();
+		bool HasHeapSummaryData {get;}
+		void HeapSummaryDataProcessed ();
+		bool HasLockContentionData {get;}
+		void LockContentionDataProcessed ();
+		
 		void Start (uint version, string runtimeFile, ProfilerFlags flags, ulong startCounter, DateTime startTime);
 		void End (uint version, ulong endCounter, DateTime endTime);
 		
 		void StartBlock (ulong startCounter, DateTime startTime, ulong threadId);
 		void EndBlock (ulong endCounter, DateTime endTime, ulong threadId);
 		
-		void ModuleLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success);
-		void ModuleUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name);
-		void AssemblyLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success);
-		void AssemblyUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name);
-		void ApplicationDomainLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success);
-		void ApplicationDomainUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name);
+		void ModuleLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success);
+		void ModuleUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name);
+		void AssemblyLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success);
+		void AssemblyUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name);
+		void ApplicationDomainLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success);
+		void ApplicationDomainUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name);
 		
 		void SetCurrentThread (ulong threadId);
 		
@@ -194,6 +302,8 @@ namespace Mono.Profiler {
 		void ClassAllocationSummary (LC c, uint reachableInstances, uint reachableBytes, uint unreachableInstances, uint unreachableBytes);
 		void AllocationSummaryEnd (uint collection, ulong endCounter, DateTime endTime);
 		
+		void MonitorEvent (MonitorEvent eventCode, LC loadedClass, ulong objectId, ulong counter);
+		
 		void InitializeData (byte[] data, uint currentOffset);
 		void DataProcessed (uint offset);
 	}
@@ -219,6 +329,13 @@ namespace Mono.Profiler {
 		}
 	}
 	public class BaseLoadedClass : BaseLoadedElement, ILoadedClass {
+		LoadedAssembly assembly;
+		public LoadedAssembly Assembly {
+			get {
+				return assembly;
+			}
+		}
+		
 		uint size;
 		public uint Size {
 			get {
@@ -226,7 +343,8 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		public BaseLoadedClass (uint id, string name, uint size) : base (id, name) {
+		public BaseLoadedClass (uint id, LoadedAssembly assembly, string name, uint size) : base (id, name) {
+			this.assembly = assembly;
 			this.size = size;
 		}
 	}
@@ -238,8 +356,16 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		public BaseLoadedMethod (uint id, LC c, string name) : base (id, name) {
+		bool isWrapper;
+		public bool IsWrapper {
+			get {
+				return isWrapper;
+			}
+		}
+		
+		public BaseLoadedMethod (uint id, LC c, bool isWrapper, string name) : base (id, name) {
 			this.c = c;
+			this.isWrapper = isWrapper;
 		}
 	}
 	public class BaseUnmanagedFunctionFromRegion<UFR> : IUnmanagedFunctionFromRegion<UFR> where UFR : IUnmanagedFunctionFromRegion<UFR> {
@@ -667,6 +793,9 @@ namespace Mono.Profiler {
 		ALLOCATIONS_CARRY_CALLER = 1,
 		ALLOCATIONS_HAVE_STACK = 2,
 		ALLOCATIONS_CARRY_ID = 3,
+		LOADED_ELEMENTS_CARRY_ID = 4,
+		CLASSES_CARRY_ASSEMBLY_ID = 5,
+		METHODS_CARRY_WRAPPER_FLAG = 6,
 		LAST
 	}
 	
@@ -701,6 +830,36 @@ namespace Mono.Profiler {
 			allocationsCarryId = true;
 		}
 		
+		bool loadedElementsCarryId;
+		public bool LoadedElementsCarryId {
+			get {
+				return loadedElementsCarryId;
+			}
+		}
+		public void LoadedElementsCarryIdReceived () {
+			loadedElementsCarryId = true;
+		}
+		
+		bool classesCarryAssemblyId;
+		public bool ClassesCarryAssemblyId {
+			get {
+				return classesCarryAssemblyId;
+			}
+		}
+		public void ClassesCarryAssemblyIdReceived () {
+			classesCarryAssemblyId = true;
+		}
+		
+		bool methodsCarryWrapperFlag;
+		public bool MethodsCarryWrapperFlag {
+			get {
+				return methodsCarryWrapperFlag;
+			}
+		}
+		public void MethodsCarryWrapperFlagReceived () {
+			methodsCarryWrapperFlag = true;
+		}
+		
 		public DirectivesHandler () {
 			allocationsCarryCallerMethod = false;
 			allocationsHaveStackTrace = false;
@@ -740,18 +899,98 @@ namespace Mono.Profiler {
 			this.directives = baseHandler.Directives;
 		}
 		
+		bool hasAllocationData;
+		public bool HasAllocationData {
+			get {
+				return hasAllocationData;
+			}
+		}
+		public void AllocationDataProcessed () {
+			hasAllocationData = true;
+		}
+		
+		bool hasCallData;
+		public bool HasCallData {
+			get {
+				return hasCallData;
+			}
+		}
+		public void CallDataProcessed () {
+			hasCallData = true;
+		}
+		
+		bool hasStatisticalData;
+		public bool HasStatisticalData {
+			get {
+				return hasStatisticalData;
+			}
+		}
+		public void StatisticalDataProcessed () {
+			hasStatisticalData = true;
+		}
+		
+		bool hasHeapSnapshotData;
+		public bool HasHeapSnapshotData {
+			get {
+				return hasHeapSnapshotData;
+			}
+		}
+		public void HeapSnapshotDataProcessed () {
+			hasHeapSnapshotData = true;
+		}
+		
+		bool hasJitTimeData;
+		public bool HasJitTimeData {
+			get {
+				return hasJitTimeData;
+			}
+		}
+		public void JitTimeDataProcessed () {
+			hasJitTimeData = true;
+		}
+		
+		bool hasGcTimeData;
+		public bool HasGcTimeData {
+			get {
+				return hasGcTimeData;
+			}
+		}
+		public void GcTimeDataProcessed () {
+			hasGcTimeData = true;
+		}
+		
+		bool hasHeapSummaryData;
+		public bool HasHeapSummaryData {
+			get {
+				return hasHeapSummaryData;
+			}
+		}
+		public void HeapSummaryDataProcessed () {
+			hasHeapSummaryData = true;
+		}
+		
+		bool hasLockContentionData;
+		public bool HasLockContentionData {
+			get {
+				return hasLockContentionData;
+			}
+		}
+		public void LockContentionDataProcessed () {
+			hasLockContentionData = true;
+		}
+		
 		public virtual void Start (uint version, string runtimeFile, ProfilerFlags flags, ulong startCounter, DateTime startTime) {}
 		public virtual void End (uint version, ulong endCounter, DateTime endTime) {}
 		
 		public virtual void StartBlock (ulong startCounter, DateTime startTime, ulong threadId) {}
 		public virtual void EndBlock (ulong endCounter, DateTime endTime, ulong threadId) {}
 		
-		public virtual void ModuleLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {}
-		public virtual void ModuleUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {}
-		public virtual void AssemblyLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {}
-		public virtual void AssemblyUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {}
-		public virtual void ApplicationDomainLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {}
-		public virtual void ApplicationDomainUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {}
+		public virtual void ModuleLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {}
+		public virtual void ModuleUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name) {}
+		public virtual void AssemblyLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {}
+		public virtual void AssemblyUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name) {}
+		public virtual void ApplicationDomainLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {}
+		public virtual void ApplicationDomainUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name) {}
 		
 		public virtual void SetCurrentThread (ulong threadId) {}
 		
@@ -803,6 +1042,8 @@ namespace Mono.Profiler {
 		public virtual void ClassAllocationSummary (LC c, uint reachableInstances, uint reachableBytes, uint unreachableInstances, uint unreachableBytes) {}
 		public virtual void AllocationSummaryEnd (uint collection, ulong endCounter, DateTime endTime) {}
 		
+		public virtual void MonitorEvent (MonitorEvent eventCode, LC loadedClass, ulong objectId, ulong counter) {}
+		
 		public virtual void InitializeData (byte[] data, uint currentOffset) {}
 		public virtual void DataProcessed (uint offset) {}
 	}
@@ -838,22 +1079,22 @@ namespace Mono.Profiler {
 			output.WriteLine ("StartBlock: endCounter {0}, endTime {1}, threadId {2}", endCounter, endTime, threadId);
 		}
 		
-		public override void ModuleLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+		public override void ModuleLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {
 			output.WriteLine ("ModuleLoaded");
 		}
-		public override void ModuleUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+		public override void ModuleUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name) {
 			output.WriteLine ("ModuleUnloaded");
 		}
-		public override void AssemblyLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+		public override void AssemblyLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {
 			output.WriteLine ("AssemblyLoaded");
 		}
-		public override void AssemblyUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+		public override void AssemblyUnloaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name) {
 			output.WriteLine ("AssemblyUnloaded");
 		}
-		public override void ApplicationDomainLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+		public override void ApplicationDomainLoaded (ulong threadId, uint id, ulong startCounter, ulong endCounter, string name, bool success) {
 			output.WriteLine ("ApplicationDomainLoaded");
 		}
-		public override void ApplicationDomainUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+		public override void ApplicationDomainUnloaded (ulong threadId,uint id,  ulong startCounter, ulong endCounter, string name) {
 			output.WriteLine ("ApplicationDomainUnloaded");
 		}
 		
@@ -1102,6 +1343,27 @@ namespace Mono.Profiler {
 	public class LoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
 		ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> factory;
 		
+		int loadedAssembliesCount;
+		LoadedAssembly[] loadedAssemblies;
+		public LoadedAssembly[] Assemblies {
+			get {
+				LoadedAssembly[] result = new LoadedAssembly [loadedAssembliesCount];
+				int resultIndex = 0;
+				for (int i = 0; i < loadedAssemblies.Length; i++) {
+					LoadedAssembly c = loadedAssemblies [i];
+					if (c != null) {
+						result [resultIndex] = c;
+						resultIndex ++;
+					}
+				}
+				return result;
+			}
+		}
+		public LoadedAssembly GetAssembly (uint id) {
+			LoadedAssembly result = loadedAssemblies [(int) id];
+			return (result != null) ? result : LoadedAssembly.Unavailable;
+		}
+		
 		int loadedClassesCount;
 		LC[] loadedClasses;
 		public LC[] Classes {
@@ -1200,8 +1462,20 @@ namespace Mono.Profiler {
 				sortedMemoryRegions.Sort (CompareByStartAddress);
 		}
 		
-		public LC NewClass (uint id, string name, uint size) {
-			LC result = factory.NewClass (id, name, size);
+		public LoadedAssembly NewAssembly (uint id, string name, string baseName, uint major, uint minor, uint build, uint revision, string culture, string publicKeyToken, bool retargetable) {
+			LoadedAssembly result = factory.NewAssembly (id, name, baseName, major, minor, build, revision, culture, publicKeyToken, retargetable);
+			if (loadedAssemblies.Length <= id) {
+				LoadedAssembly[] newLoadedAssemblies = new LoadedAssembly [((int) id + 1) * 2];
+				loadedAssemblies.CopyTo (newLoadedAssemblies, 0);
+				loadedAssemblies = newLoadedAssemblies;
+			}
+			loadedAssemblies [(int) id] = result;
+			loadedAssembliesCount ++;
+			return result;
+		}
+		
+		public LC NewClass (uint id, LoadedAssembly assembly, string name, uint size) {
+			LC result = factory.NewClass (id, assembly, name, size);
 			if (loadedClasses.Length <= id) {
 				LC[] newLoadedClasses = new LC [((int) id + 1) * 2];
 				loadedClasses.CopyTo (newLoadedClasses, 0);
@@ -1212,8 +1486,8 @@ namespace Mono.Profiler {
 			return result;
 		}
 		
-		public LM NewMethod (uint id, LC c, string name) {
-			LM result = factory.NewMethod (id, c, name);
+		public LM NewMethod (uint id, LC c, bool isWrapper, string name) {
+			LM result = factory.NewMethod (id, c, isWrapper, name);
 			if (loadedMethods.Length <= id) {
 				LM[] newLoadedMethods = new LM [((int) id + 1) * 2];
 				loadedMethods.CopyTo (newLoadedMethods, 0);
@@ -1287,6 +1561,7 @@ namespace Mono.Profiler {
 		
 		public LoadedElementHandler (ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> factory) {
 			this.factory = factory;
+			loadedAssemblies = new LoadedAssembly [10];
 			loadedClasses = new LC [1000];
 			loadedClassesCount = 0;
 			loadedMethods = new LM [5000];
