@@ -21,6 +21,10 @@ using System.Xml;
 
 using Mono.Options;
 
+#if MACOS
+using OSXIntegration.Framework;
+#endif
+
 namespace Monodoc {
 class Driver {
 	  
@@ -56,7 +60,7 @@ class Driver {
 				v => sources.Add (v) },
 			{ "edit=",
 				"Edit mdoc(5) XML documentation found within {PATH}.",
-				v => RootTree.UncompiledHelpSources.Add (v) },
+				v => RootTree.AddUncompiledSource (v) },
 			{ "engine=",
 				"Specify which HTML rendering {ENGINE} to use:\n" + 
 					"  " + string.Join ("\n  ", engines) + "\n" +
@@ -115,6 +119,9 @@ class Driver {
 
 		List<string> topics = p.Parse (args);
 
+		if (basedir == null)
+			basedir = Directory.GetParent (System.Reflection.Assembly.GetExecutingAssembly ().Location).FullName;
+
 		if (show_version) {
 			Console.WriteLine ("Mono Documentation Browser");
 			Version ver = Assembly.GetExecutingAssembly ().GetName ().Version;
@@ -128,7 +135,7 @@ class Driver {
 			return r;
 		}
 
-		if (mergeConfigFile != null) {
+		/*if (mergeConfigFile != null) {
 			ArrayList targetDirs = new ArrayList ();
 			
 			for (int i = 0; i < topics.Count; i++)
@@ -141,7 +148,7 @@ class Driver {
 
 			e.Merge ();
 			return 0;
-		}
+		}*/
 		
 		if (r != 0 || !show_gui)
 			return r;
@@ -253,6 +260,7 @@ public class Browser {
 	TreeView search_tree;
 	TreeStore search_store;
 	SearchableIndex search_index;
+	ArrayList searchResults = new ArrayList (20);
 	string highlight_text;
 	[Glade.Widget] VBox search_vbox;
 	ProgressPanel ppanel;
@@ -287,6 +295,14 @@ public class Browser {
 
 	public Browser (string basedir, IEnumerable<string> sources, string engine)
 	{
+#if MACOS
+		try {
+			InitMacAppHandlers();
+		} catch (Exception ex) {
+			Console.Error.WriteLine ("Installing Mac AppleEvent handlers failed. Skipping.\n" + ex);
+		}
+#endif
+	
 		this.engine = engine;		
 		ui = new Glade.XML (null, "browser.glade", "window1", null);
 		ui.Autoconnect (this);
@@ -390,11 +406,61 @@ public class Browser {
 		bookList = new ArrayList ();
 
 		index_browser = IndexBrowser.MakeIndexBrowser (this);
-		
 		MainWindow.ShowAll();
+		
+#if MACOS
+		try {
+			InstallMacMainMenu ();
+			((MenuBar)ui["menubar1"]).Hide ();
+		} catch (Exception ex) {
+			Console.Error.WriteLine ("Installing Mac IGE Main Menu failed. Skipping.\n" + ex);
+		}
+#endif
 	}
 
+#if MACOS
+		void InstallMacMainMenu ()
+		{
+			IgeMacIntegration.IgeMacMenu.GlobalKeyHandlerEnabled = true;
+			IgeMacIntegration.IgeMacMenu.MenuBar = (MenuBar) ui["menubar1"];
+			IgeMacIntegration.IgeMacMenu.QuitMenuItem = (MenuItem) ui["quit1"];
+			var appGroup = IgeMacIntegration.IgeMacMenu.AddAppMenuGroup ();
+			appGroup.AddMenuItem ((MenuItem)ui["quit1"], null);
+			appGroup.AddMenuItem ((MenuItem)ui["about1"], null);
+		}
 
+		void InitMacAppHandlers ()
+		{
+			ApplicationEvents.Quit += delegate (object sender, ApplicationEventArgs e) {
+				Application.Quit ();
+				e.Handled = true;
+			};
+			
+			ApplicationEvents.Reopen += delegate (object sender, ApplicationEventArgs e) {
+				if (MainWindow != null) {
+					MainWindow.Deiconify ();
+					MainWindow.Visible = true;
+					e.Handled = true;
+				}
+			};
+			
+			ApplicationEvents.OpenUrls += delegate (object sender, ApplicationUrlEventArgs e) {
+				if (e.Urls == null || e.Urls.Count == 0)
+					return;
+				string url = e.Urls[0];
+				if (string.IsNullOrEmpty (url) || !url.StartsWith ("monodoc://"))
+					return;
+				url = url.Substring ("monodoc://".Length);
+				if (url.Length == 0)
+					return;
+				url = System.Web.HttpUtility.UrlDecode (url);
+				LoadUrl (url);
+				if (MainWindow != null)
+					MainWindow.Present ();
+				e.Handled = true;
+			};
+		}
+#endif
 
 	// Initianlizes the search index
 	void CreateSearchPanel ()
@@ -516,6 +582,7 @@ public class Browser {
 		Result r = search_index.Search (term);
 		if (r == null)
 			return; //There was a problem with the index
+		searchResults.Add (r);
 		//insert the results in the tree
 		TreeIter iter;
 					
@@ -560,7 +627,7 @@ public class Browser {
 			return;
 		int i_0 = p.Indices [0];
 		int i_1 = p.Indices [1];
-		Result res = (Result) search_index.Results [i_0];
+		Result res = (Result) searchResults [i_0];
 		TreeIter parent;
 		model.IterParent (out parent, iter);
 		string term = (string) search_store.GetValue (parent, 0);
@@ -586,21 +653,21 @@ public class Browser {
 	void TextLarger (object obj, EventArgs args)
 	{
 		SettingsHandler.Settings.preferred_font_size += 10;
-		HelpSource.CssCode = null;
+		//HelpSource.CssCode = null;
 		Reload ();
 		SettingsHandler.Save ();
 	}
 	void TextSmaller (object obj, EventArgs args)
 	{
 		SettingsHandler.Settings.preferred_font_size -= 10;
-		HelpSource.CssCode = null;
+		//HelpSource.CssCode = null;
 		Reload ();
 		SettingsHandler.Save ();
 	}
 	void TextNormal (object obj, EventArgs args)
 	{
 		SettingsHandler.Settings.preferred_font_size = 100;
-		HelpSource.CssCode = null;
+		//HelpSource.CssCode = null;
 		Reload ();
 		SettingsHandler.Save ();
 	}
@@ -671,6 +738,15 @@ public class Browser {
 		
 		Node node;
 		
+		/*
+		 * The webkit library converts the url titles (N:, T:, etc.) to lower case (n:, t:, etc.)
+		 * when clicking on a link. Therefore we need to convert them to upper case, since the
+		 * monodoc backend only understands upper case titles (except for root:, afaik).
+		 */
+		string[] urlParts = url.Split (':');
+		if (urlParts [0].Length == 1)
+			url = urlParts [0].ToUpper () + url.Substring (1);
+			
 		Console.Error.WriteLine ("Trying: {0}", url);
 		try {
 			string res = Browser.GetHtml (url, null, help_tree, out node);
@@ -723,8 +799,8 @@ public class Browser {
 			//
 			string tabTitle;
 			tabTitle = matched_node.Caption; //Normal title
-			string[] parts = matched_node.URL.Split('/', '#');
-			if(matched_node.URL != null && matched_node.URL.StartsWith("ecma:")) {
+			string[] parts = matched_node.PublicUrl.Split('/', '#');
+			if(matched_node.PublicUrl != null && matched_node.PublicUrl.StartsWith("ecma:")) {
 				if(parts.Length == 3 && parts[2] != String.Empty) { //List of Members, properties, events, ...
 					tabTitle = parts[1] + ": " + matched_node.Caption;
 				} else if(parts.Length >= 4) { //Showing a concrete Member, property, ...					
@@ -875,6 +951,7 @@ ExtLoop:
 	void delete_event_cb (object o, DeleteEventArgs args)
 	{
 		Application.Quit ();
+		args.RetVal = true;
 	}
 	void on_print_activate (object sender, EventArgs e) 
 	{
@@ -1629,7 +1706,7 @@ ExtLoop:
 
 		void OnOkClicked (object sender, EventArgs a)
 		{
-			CommentService service = new CommentService();
+			//CommentService service = new CommentService();
 			// todo
 			newcomment.Hide ();
 		}
@@ -1948,7 +2025,7 @@ public class TreeBrowser {
 		if (tree_view.Selection.GetSelected (out model, out iter)){
 			Node n = (Node) iter_to_node [iter];
 			
-			string url = n.URL;
+			string url = n.PublicUrl;
 			Node match;
 			string s;
 
@@ -1976,7 +2053,7 @@ public class TreeBrowser {
 				return;
 			}
 
-			((Browser)browser).Render ("<h1>Unhandled URL</h1>" + "<p>Functionality to view the resource <i>" + n.URL + "</i> is not available on your system or has not yet been implemented.</p>", null, url);
+			((Browser)browser).Render ("<h1>Unhandled URL</h1>" + "<p>Functionality to view the resource <i>" + n.PublicUrl + "</i> is not available on your system or has not yet been implemented.</p>", null, url);
 		}
 	}
 }
@@ -2366,7 +2443,15 @@ public class Tab : Notebook {
 		return null;
 	}
 	
-	
+#if MACOS
+	public static IHtmlRender GetRenderer (string engine, Browser browser)
+	{
+		var renderer = new GtkHtmlHtmlRender (browser.help_tree);
+		renderer.Initialize ();
+
+		return renderer;
+	}
+#else
 	public static IHtmlRender GetRenderer (string engine, Browser browser)
 	{
 		IHtmlRender renderer = LoadRenderer (System.IO.Path.Combine (AppDomain.CurrentDomain.BaseDirectory, engine + "HtmlRender.dll"), browser);
@@ -2399,7 +2484,7 @@ public class Tab : Notebook {
 		
 		return null;		
 	}
-	
+#endif
 
 	public Tab(Browser br) 
 	{
@@ -2632,7 +2717,7 @@ public class Tab : Notebook {
 		string [] uSplit = EditingUtils.ParseEditUrl (edit_url);
 		
 		if (uSplit[0].StartsWith ("monodoc:"))
-			EditingUtils.SaveChange (edit_url, browser.help_tree, edit_node, EcmaHelpSource.GetNiceUrl (browser.CurrentTab.CurrentNode));
+			EditingUtils.SaveChange (edit_url, browser.help_tree, edit_node, GetNiceUrl (browser.CurrentTab.CurrentNode));
 		else if (uSplit[0].StartsWith ("file:"))
 			EditingUtils.SaveChange (edit_url, browser.help_tree, edit_node, String.Empty);
 		else
@@ -2641,6 +2726,49 @@ public class Tab : Notebook {
 		history.ActivateCurrent ();
 	}
 
+	public static string GetNiceUrl (Node node) {
+		if (node.Element.StartsWith("N:"))
+			return node.Element;
+		string name, full;
+		int bk_pos = node.Caption.IndexOf (' ');
+		// node from an overview
+		if (bk_pos != -1) {
+			name = node.Caption.Substring (0, bk_pos);
+			full = node.Parent.Caption + "." + name.Replace ('.', '+');
+			return "T:" + full;
+		}
+		// node that lists constructors, methods, fields, ...
+		if ((node.Caption == "Constructors") || (node.Caption == "Fields") || (node.Caption == "Events") 
+			|| (node.Caption == "Members") || (node.Caption == "Properties") || (node.Caption == "Methods")
+			|| (node.Caption == "Operators")) {
+			bk_pos = node.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			return "T:" + full + "/" + node.Element; 
+		}
+		int pr_pos = node.Caption.IndexOf ('(');
+		// node from a constructor
+		if (node.Parent.Element == "C") {
+			name = node.Parent.Parent.Parent.Caption;
+			int idx = node.PublicUrl.IndexOf ('/');
+			return node.PublicUrl[idx+1] + ":" + name + "." + node.Caption.Replace ('.', '+');
+		// node from a method with one signature, field, property, operator
+		} else if (pr_pos == -1) {
+			bk_pos = node.Parent.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			int idx = node.PublicUrl.IndexOf ('/');
+			return node.PublicUrl[idx+1] + ":" + full + "." + node.Caption;
+		// node from a method with several signatures
+		} else {
+			bk_pos = node.Parent.Parent.Parent.Caption.IndexOf (' ');
+			name = node.Parent.Parent.Parent.Caption.Substring (0, bk_pos);
+			full = node.Parent.Parent.Parent.Parent.Caption + "." + name.Replace ('.', '+');
+			int idx = node.PublicUrl.IndexOf ('/');
+			return node.PublicUrl[idx+1] + ":" + full + "." + node.Caption;
+		}
+	}
+
 	void OnCancelEdits (object sender, EventArgs a)
 	{
 		SetMode (Mode.Viewer);
@@ -2667,6 +2795,7 @@ public class Tab : Notebook {
 			
 			StringWriter sw = new StringWriter ();
 			XmlWriter w = new XmlTextWriter (sw);
+			var converter = new Monodoc.Generators.Html.Ecma2Html ();
 			
 			try {
 				edit_node.InnerXml = text_editor.Buffer.Text;
@@ -2680,7 +2809,7 @@ public class Tab : Notebook {
 			}
 			browser.statusbar.Pop (browser.context_id);
 			browser.statusbar.Push (browser.context_id, "XML OK");
-			string s = HelpSource.BuildHtml (EcmaHelpSource.css_ecma_code, sw.ToString ());
+			string s = converter.Export (sw.ToString (), new Dictionary<string, string> ());
 			html_preview.Render(s);
 
 			return false;
