@@ -97,9 +97,9 @@ namespace Gendarme.Framework.Helpers {
 				return this == other;
 			}
 
-			public bool Equals (StoreSlot storeSlot)
+			public bool Equals (StoreSlot other)
 			{
-				return this == storeSlot;
+				return this == other;
 			}
 
 			public override int GetHashCode ()
@@ -182,40 +182,14 @@ namespace Gendarme.Framework.Helpers {
 				return false;
 			}
 
-			public bool Equals (InstructionWithLeave iwl)
+			public bool Equals (InstructionWithLeave other)
 			{
-				if (Instruction != iwl.Instruction)
-					return false;
-
-				if (LeaveStack == null)
-					return (iwl.LeaveStack == null);
-
-				if (iwl.LeaveStack == null)
-					return false;
-
-				if (LeaveStack.Length != iwl.LeaveStack.Length)
-					return false;
-
-				for (int i = 0; i < LeaveStack.Length; i++) {
-					if (LeaveStack [i] != iwl.LeaveStack [i])
-						return false;
-				}
-				return true;
+				return (Instruction == other.Instruction);
 			}
 
 			public override int GetHashCode ()
 			{
-				int hc = 0;
-				
-				unchecked {
-					hc ^= Instruction.GetHashCode ();
-					if (LeaveStack != null) {
-						foreach (Instruction ins in LeaveStack)
-							hc ^= ins.GetHashCode ();
-					}
-				}
-				
-				return hc;
+				return Instruction.GetHashCode ();
 			}
 
 			public static bool operator == (InstructionWithLeave left, InstructionWithLeave right)
@@ -253,6 +227,9 @@ namespace Gendarme.Framework.Helpers {
 		/// <returns>An array of UsageResults containing the instructions that use the value and the stack offset of the entry at that instruction. A StackOffset of 0 means right on top of the stack.</returns>
 		public StackEntryUsageResult [] GetStackEntryUsage (Instruction ins)
 		{
+			if (ins == null)
+				throw new ArgumentNullException ("ins");
+
 			/* In the main loop we search for all usages of a StackEntry.
 			 * Then we check each usage for a store (to a local variable or an argument), search for corrosponding loads and search for usages of the new Stackentry.
 			 * This continues until no stores are found. */
@@ -398,25 +375,31 @@ namespace Gendarme.Framework.Helpers {
 						LoadResults.AddIfNew (insWithLeave.Copy (ins)); //continue, might be loaded again
 
 					//we simply branch to every possible catch block.
-					foreach (ExceptionHandler handler in Body.ExceptionHandlers) {
-						if (handler.Type != ExceptionHandlerType.Catch)
-							continue;
-						if (ins.Offset < handler.TryStart.Offset || ins.Offset >= handler.TryEnd.Offset)
-							continue;
-						LoadAlternatives.AddIfNew (insWithLeave.Copy (handler.HandlerStart));
+					IList<ExceptionHandler> ehc = null;
+					if (Body.HasExceptionHandlers) {
+						ehc = Body.ExceptionHandlers;
+						foreach (ExceptionHandler handler in ehc) {
+							if (handler.HandlerType != ExceptionHandlerType.Catch)
+								continue;
+							if (ins.Offset < handler.TryStart.Offset || ins.Offset >= handler.TryEnd.Offset)
+								continue;
+							LoadAlternatives.AddIfNew (insWithLeave.Copy (handler.HandlerStart));
+						}
 					}
 
 					//Code.Leave leaves a try/catch block. Search for the finally block.
 					if (ins.OpCode.Code == Code.Leave || ins.OpCode.Code == Code.Leave_S) {
 						bool handlerFound = false;
-						foreach (ExceptionHandler handler in Body.ExceptionHandlers) {
-							if (handler.Type != ExceptionHandlerType.Finally)
-								continue;
-							if (handler.TryStart.Offset > ins.Offset || handler.TryEnd.Offset <= ins.Offset)
-								continue;
-							LoadAlternatives.AddIfNew (insWithLeave.Push (handler.HandlerStart, ins)); //push the leave instruction onto the leave stack
-							handlerFound = true;
-							break;
+						if (ehc != null) {
+							foreach (ExceptionHandler handler in ehc) {
+								if (handler.HandlerType != ExceptionHandlerType.Finally)
+									continue;
+								if (handler.TryStart.Offset > ins.Offset || handler.TryEnd.Offset <= ins.Offset)
+									continue;
+								LoadAlternatives.AddIfNew (insWithLeave.Push (handler.HandlerStart, ins)); //push the leave instruction onto the leave stack
+								handlerFound = true;
+								break;
+							}
 						}
 						if (!handlerFound) //no finally found (try/catch without finally)
 							LoadAlternatives.AddIfNew (insWithLeave.Copy ((Instruction) ins.Operand));
@@ -464,6 +447,9 @@ namespace Gendarme.Framework.Helpers {
 		/// <returns>The next instruction that would be executed by the runtime.</returns>
 		public static Instruction GetNextInstruction (Instruction ins, out object alternative)
 		{
+			if (ins == null)
+				throw new ArgumentNullException ("ins");
+
 			alternative = null;
 			switch (ins.OpCode.FlowControl) {
 			case FlowControl.Branch:
@@ -480,7 +466,7 @@ namespace Gendarme.Framework.Helpers {
 			case FlowControl.Throw:
 				return null;
 			default:
-				throw new NotImplementedException ("FlowControl: " + ins.OpCode.FlowControl + " is not supported.");
+				throw new NotImplementedException ("FlowControl: " + ins.OpCode.FlowControl.ToString () + " is not supported.");
 
 			}
 		}
@@ -504,9 +490,9 @@ namespace Gendarme.Framework.Helpers {
 
 			case Code.Ldfld:
 				//TODO: we do not check what instance is on the stack
-				return new StoreSlot (StoreType.Field, (int) ((FieldReference) ins.Operand).MetadataToken.ToUInt ());
+				return new StoreSlot (StoreType.Field, ((FieldReference) ins.Operand).MetadataToken.ToInt32 ());
 			case Code.Ldsfld:
-				return new StoreSlot (StoreType.StaticField, (int) ((FieldReference) ins.Operand).MetadataToken.ToUInt ());
+				return new StoreSlot (StoreType.StaticField, ((FieldReference) ins.Operand).MetadataToken.ToInt32 ());
 
 			case Code.Ldarg_0:
 			case Code.Ldarg_1:
@@ -515,7 +501,7 @@ namespace Gendarme.Framework.Helpers {
 				return new StoreSlot (StoreType.Argument, ins.OpCode.Code - Code.Ldarg_0);
 			case Code.Ldarg_S:
 			case Code.Ldarg: {
-					int sequence = ((ParameterDefinition) ins.Operand).Sequence;
+					int sequence = ((ParameterDefinition) ins.Operand).Index + 1;
 					if (!this.Method.HasThis)
 						sequence--;
 					return new StoreSlot (StoreType.Argument, sequence);
@@ -563,13 +549,13 @@ namespace Gendarme.Framework.Helpers {
 
 			case Code.Stfld:
 				//TODO: we do not check what instance is on the stack
-				return new StoreSlot (StoreType.Field, (int) ((FieldReference) ins.Operand).MetadataToken.ToUInt ());
+				return new StoreSlot (StoreType.Field, ((FieldReference) ins.Operand).MetadataToken.ToInt32 ());
 			case Code.Stsfld:
-				return new StoreSlot (StoreType.StaticField, (int) ((FieldReference) ins.Operand).MetadataToken.ToUInt ());
+				return new StoreSlot (StoreType.StaticField, ((FieldReference) ins.Operand).MetadataToken.ToInt32 ());
 
 			case Code.Starg_S: //store arg (not ref / out etc)
 			case Code.Starg: {
-					int sequence = ((ParameterDefinition) ins.Operand).Sequence;
+					int sequence = ((ParameterDefinition) ins.Operand).Index + 1;
 					if (!this.Method.HasThis)
 						sequence--;
 					return new StoreSlot (StoreType.Argument, sequence);
