@@ -56,26 +56,6 @@ namespace Gendarme.Framework.Rocks {
 	/// </summary>
 	public static class TypeRocks {
 
-		/// <summary>
-		/// Return an IEnumerable that allows a single loop (like a foreach) to
-		/// traverse all MethodDefinition in the type. That includes the Constructors
-		/// and Methods collections.
-		/// </summary>
-		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <returns>An IEnumerable to traverse all constructors and methods</returns>
-		public static IEnumerable<MethodDefinition> AllMethods (this TypeReference self)
-		{
-			TypeDefinition type = self.Resolve ();
-			if (type.HasConstructors) {
-				foreach (MethodDefinition ctor in type.Constructors)
-					yield return ctor;
-			}
-			if (type.HasMethods) {
-				foreach (MethodDefinition method in type.Methods)
-					yield return method;
-			}
-		}
-
 		/// <summary>
 		/// Returns an IEnumerable that allows a single loop (like a foreach) to
 		/// traverse all base classes and interfaces inherited by the type.
@@ -103,47 +83,6 @@ namespace Gendarme.Framework.Rocks {
 			}
 		}
 
-		/// <summary>
-		/// Check if a type reference collection contains a type of a specific name.
-		/// </summary>
-		/// <param name="self">The TypeReferenceCollection on which the extension method can be called.</param>
-		/// <param name="typeName">Full name of the type.</param>
-		/// <returns>True if the collection contains an type of the same name,
-		/// False otherwise.</returns>
-		public static bool ContainsType (this TypeReferenceCollection self, string typeName)
-		{
-			if (typeName == null)
-				throw new ArgumentNullException ("typeName");
-
-			foreach (TypeReference type in self) {
-				if (type.FullName == typeName)
-					return true;
-			}
-			return false;
-		}
-
-		/// <summary>
-		/// Check if a type reference collection contains any of the specified type names.
-		/// </summary>
-		/// <param name="self">The TypeReferenceCollection on which the extension method can be called.</param>
-		/// <param name="typeNames">A string array of full type names.</param>
-		/// <returns>True if the collection contains any types matching one specified,
-		/// False otherwise.</returns>
-		public static bool ContainsAnyType (this TypeReferenceCollection self, string [] typeNames)
-		{
-			if (typeNames == null)
-				throw new ArgumentNullException ("typeNames");
-
-			foreach (TypeReference type in self) {
-				string fullname = type.FullName;
-				foreach (string type_full_name in typeNames) {
-					if (fullname == type_full_name)
-						return true;
-				}
-			}
-			return false;
-		}
-
 		/// <summary>
 		/// Returns the first MethodDefinition that satisfies a given MethodSignature.
 		/// </summary>
@@ -155,27 +94,16 @@ namespace Gendarme.Framework.Rocks {
 		/// </remarks>
 		public static MethodDefinition GetMethod (this TypeReference self, MethodSignature signature)
 		{
-			bool ctors, methods;
-			// method name is optional so we must look in everything
-			if (String.IsNullOrEmpty (signature.Name)) {
-				ctors = true;
-				methods = true;
-			} else {
-				ctors = (signature.Name [0] == '.');
-				methods = !ctors;
-			}
-			
+			if (signature == null)
+				throw new ArgumentNullException ("signature");
+			if (self == null)
+				return null;
+
 			TypeDefinition type = self.Resolve ();
 			if (type == null)
 				return null;
 
-			if (ctors && type.HasConstructors) {
-				foreach (MethodDefinition ctor in type.Constructors) {
-					if (signature.Matches (ctor))
-						return ctor;
-				}
-			}
-			if (methods && type.HasMethods) {
+			if (type.HasMethods) {
 				foreach (MethodDefinition method in type.Methods) {
 					if (signature.Matches (method))
 						return method;
@@ -196,22 +124,26 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>The first MethodDefinition that satisfies all conditions.</returns>
 		public static MethodDefinition GetMethod (this TypeReference self, MethodAttributes attributes, string name, string returnType, string [] parameters, Func<MethodDefinition, bool> customCondition)
 		{
-			foreach (MethodDefinition method in self.AllMethods ()) {
+			if (self == null)
+				return null;
+
+			foreach (MethodDefinition method in self.Resolve ().Methods) {
 				if (name != null && method.Name != name)
 					continue;
 				if ((method.Attributes & attributes) != attributes)
 					continue;
-				if (returnType != null && method.ReturnType.ReturnType.FullName != returnType)
+				if (returnType != null && !method.ReturnType.IsNamed (returnType))
 					continue;
 				if (parameters != null) {
 					if (method.HasParameters) {
-						if (parameters.Length != method.Parameters.Count)
+						IList<ParameterDefinition> pdc = method.Parameters;
+						if (parameters.Length != pdc.Count)
 							continue;
 						bool parameterError = false;
 						for (int i = 0; i < parameters.Length; i++) {
 							if (parameters [i] == null)
 								continue;//ignore parameter
-							if (parameters [i] != method.Parameters [i].ParameterType.GetOriginalType ().FullName) {
+							if (!pdc [i].ParameterType.GetElementType ().IsNamed (parameters [i])) {
 								parameterError = true;
 								break;
 							}
@@ -298,7 +230,7 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if at least one method matches the signature. Otherwise false.</returns>
 		public static bool HasMethod (this TypeReference self, MethodSignature signature)
 		{
-			return (self.GetMethod (signature) != null);
+			return ((self != null) && self.GetMethod (signature) != null);
 		}
 
 		/// <summary>
@@ -307,36 +239,40 @@ namespace Gendarme.Framework.Rocks {
 		/// where the information resides could be unavailable. False is returned in this case.
 		/// </summary>
 		/// <param name="self">The TypeDefinition on which the extension method can be called.</param>
-		/// <param name="interfaceName">Full name of the interface</param>
+		/// <param name="nameSpace">The namespace of the interface to be matched</param>
+		/// <param name="name">The name of the interface to be matched</param>
 		/// <returns>True if we found that the type implements the interface, False otherwise (either it
 		/// does not implement it, or we could not find where it does).</returns>
-		public static bool Implements (this TypeReference self, string interfaceName)
+		public static bool Implements (this TypeReference self, string nameSpace, string name)
 		{
-			if (interfaceName == null)
-				throw new ArgumentNullException ("interfaceName");
+			if (nameSpace == null)
+				throw new ArgumentNullException ("nameSpace");
+			if (name == null)
+				throw new ArgumentNullException ("name");
+			if (self == null)
+				return false;
 
 			TypeDefinition type = self.Resolve ();
 			if (type == null)
 				return false;	// not enough information available
 
 			// special case, check if we implement ourselves
-			if (type.IsInterface && (type.FullName == interfaceName))
+			if (type.IsInterface && type.IsNamed (nameSpace, name))
 				return true;
 
-			return Implements (type, interfaceName, (interfaceName.IndexOf ('`') >= 0));
+			return Implements (type, nameSpace, name);
 		}
 
-		private static bool Implements (TypeDefinition type, string interfaceName, bool generic)
+		private static bool Implements (TypeDefinition type, string nameSpace, string iname)
 		{
 			while (type != null) {
 				// does the type implements it itself
 				if (type.HasInterfaces) {
 					foreach (TypeReference iface in type.Interfaces) {
-						string fullname = (generic) ? iface.GetOriginalType ().FullName : iface.FullName;
-						if (fullname == interfaceName)
+						if (iface.IsNamed (nameSpace, iname))
 							return true;
 						//if not, then maybe one of its parent interfaces does
-						if (Implements (iface.Resolve (), interfaceName, generic))
+						if (Implements (iface.Resolve (), nameSpace, iname))
 							return true;
 					}
 				}
@@ -352,17 +288,24 @@ namespace Gendarme.Framework.Rocks {
 		/// where the information resides could be unavailable.
 		/// </summary>
 		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <param name="className">Full name of the base class</param>
+		/// <param name="nameSpace">The namespace of the base class to be matched</param>
+		/// <param name="name">The name of the base class to be matched</param>
 		/// <returns>True if the type inherits from specified class, False otherwise</returns>
-		public static bool Inherits (this TypeReference self, string className)
+		public static bool Inherits (this TypeReference self, string nameSpace, string name)
 		{
-			if (className == null)
-				throw new ArgumentNullException ("className");
+			if (nameSpace == null)
+				throw new ArgumentNullException ("nameSpace");
+			if (name == null)
+				throw new ArgumentNullException ("name");
+			if (self == null)
+				return false;
 
 			TypeReference current = self.Resolve ();
-			while ((current != null) && (current.FullName != "System.Object")) {
-				if (current.FullName == className)
+			while (current != null) {
+				if (current.IsNamed (nameSpace, name))
 					return true;
+				if (current.IsNamed ("System", "Object"))
+					return false;
 
 				TypeDefinition td = current.Resolve ();
 				if (td == null)
@@ -373,13 +316,67 @@ namespace Gendarme.Framework.Rocks {
 		}
 
 		/// <summary>
-		/// Check if the type represent an array (of any other type).
+		/// Check if the type and its namespace are named like the provided parameters.
+		/// This is preferred to checking the FullName property since the later can allocate (string) memory.
 		/// </summary>
 		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <returns>True if the type is an array, False otherwise</returns>
-		public static bool IsArray (this TypeReference self)
+		/// <param name="nameSpace">The namespace to be matched</param>
+		/// <param name="name">The type name to be matched</param>
+		/// <returns>True if the type is namespace and name match the arguments, False otherwise</returns>
+		public static bool IsNamed (this TypeReference self, string nameSpace, string name)
 		{
-			return (self is ArrayType);
+			if (nameSpace == null)
+				throw new ArgumentNullException ("nameSpace");
+			if (name == null)
+				throw new ArgumentNullException ("name");
+			if (self == null)
+				return false;
+
+			if (self.IsNested) {
+				int spos = name.LastIndexOf ('/');
+				if (spos == -1)
+					return false;
+				// GetFullName could be optimized away but it's a fairly uncommon case
+				return (nameSpace + "." + name == self.GetFullName ());
+			}
+
+			return ((self.Namespace == nameSpace) && (self.Name == name));
+		}
+
+		/// <summary>
+		/// Check if the type full name match the provided parameter.
+		/// Note: prefer the overload where the namespace and type name can be supplied individually
+		/// </summary>
+		/// <param name="self">The TypeReference on which the extension method can be called.</param>
+		/// <param name="fullName">The full name to be matched</param>
+		/// <returns>True if the type is namespace and name match the arguments, False otherwise</returns>
+		public static bool IsNamed (this TypeReference self, string fullName)
+		{
+			if (fullName == null)
+				throw new ArgumentNullException ("fullName");
+			if (self == null)
+				return false;
+
+			if (self.IsNested) {
+				int spos = fullName.LastIndexOf ('/');
+				if (spos == -1)
+					return false;
+				// FIXME: GetFullName could be optimized away but it's a fairly uncommon case
+				return (fullName == self.GetFullName ());
+			}
+
+			int dpos = fullName.LastIndexOf ('.');
+			string nspace = self.Namespace;
+			if (dpos != nspace.Length)
+				return false;
+
+			if (String.CompareOrdinal (nspace, 0, fullName, 0, dpos) != 0)
+				return false;
+
+			string name = self.Name;
+			if (fullName.Length - dpos - 1 != name.Length)
+				return false;
+			return (String.CompareOrdinal (name, 0, fullName, dpos + 1, fullName.Length - dpos - 1) == 0);
 		}
 
 		/// <summary>
@@ -392,7 +389,10 @@ namespace Gendarme.Framework.Rocks {
 		/// False otherwise.</returns>
 		public static bool IsAttribute (this TypeReference self)
 		{
-			return self.Inherits ("System.Attribute");
+			if (self == null)
+				return false;
+
+			return self.Inherits ("System", "Attribute");
 		}
 
 		/// <summary>
@@ -402,18 +402,19 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type is a delegate, False otherwise.</returns>
 		public static bool IsDelegate (this TypeReference self)
 		{
+			if (self == null)
+				return false;
+
 			TypeDefinition type = self.Resolve ();
 			// e.g. this occurs for <Module> or GenericParameter
 			if (null == type || type.BaseType == null)
 				return false;
 
-			switch (type.BaseType.FullName) {
-			case "System.Delegate":
-			case "System.MulticastDelegate":
-				return true;
-			default:
+			if (type.BaseType.Namespace != "System")
 				return false;
-			}
+
+			string name = type.BaseType.Name;
+			return ((name == "Delegate") || (name == "MulticastDelegate"));
 		}
 
 		/// <summary>
@@ -423,11 +424,14 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type as the [Flags] attribute, false otherwise.</returns>
 		public static bool IsFlags (this TypeReference self)
 		{
+			if (self == null)
+				return false;
+
 			TypeDefinition type = self.Resolve ();
 			if ((type == null) || !type.IsEnum || !type.HasCustomAttributes)
 				return false;
 
-			return type.HasAttribute ("System.FlagsAttribute");
+			return type.HasAttribute ("System", "FlagsAttribute");
 		}
 
 		/// <summary>
@@ -437,8 +441,14 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type is System.Single (C# float) or System.Double (C3 double), False otherwise.</returns>
 		public static bool IsFloatingPoint (this TypeReference self)
 		{
-			return ((self.FullName == Mono.Cecil.Constants.Single) ||
-				(self.FullName == Mono.Cecil.Constants.Double));
+			if (self == null)
+				return false;
+
+			if (self.Namespace != "System")
+				return false;
+
+			string name = self.Name;
+			return ((name == "Single") || (name == "Double"));
 		}
 
 		/// <summary>
@@ -449,10 +459,14 @@ namespace Gendarme.Framework.Rocks {
 		/// False otherwise (e.g. compiler or tool generated)</returns>
 		public static bool IsGeneratedCode (this TypeReference self)
 		{
-			if (self.HasCustomAttributes) {
+			if (self == null)
+				return false;
+
+			if (self.IsDefinition) {
+				TypeDefinition type = self.Resolve ();
 				// both helpful attributes only exists in 2.0 and more recent frameworks
-				if (self.Module.Assembly.Runtime >= TargetRuntime.NET_2_0) {
-					if (self.CustomAttributes.ContainsAnyType (CustomAttributeRocks.GeneratedCodeAttributes))
+				if (type.Module.Runtime >= TargetRuntime.Net_2_0) {
+					if (type.HasAnyGeneratedCodeAttribute ())
 						return true;
 				}
 			}
@@ -477,45 +491,14 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type refers to native code, False otherwise</returns>
 		public static bool IsNative (this TypeReference self)
 		{
-			switch (self.FullName) {
-			case "System.IntPtr":
-			case "System.UIntPtr":
-			case "System.Runtime.InteropServices.HandleRef":
-				return true;
-			default:
-				return false;
-			}
-		}
-
-		/// <summary>
-		/// Check if the type refers to a primitive type.
-		/// </summary>
-		/// <param name="self">The TypeReference on which the extension method can be called.</param>
-		/// <returns>True if the type is a primitive type, False otherwise</returns>
-		public static bool IsPrimitive (this TypeReference self)
-		{
-			if ((self == null) || (self.Namespace != "System"))
+			if (self == null)
 				return false;
 
-			switch (self.Name) {
-			case "Byte":
-			case "SByte":
-			case "Boolean":
-			case "Int16":
-			case "UInt16":
-			case "Char":
-			case "Int32":
-			case "UInt32":
-			case "Single":
-			case "Int64":
-			case "UInt64":
-			case "Double":
-			case "IntPtr":
-			case "UIntPtr":
-				return true;
-			default:
-				return false;
+			if (self.Namespace == "System") {
+				string name = self.Name;
+				return ((name == "IntPtr") || (name == "UIntPtr"));
 			}
+			return self.IsNamed ("System.Runtime.InteropServices", "HandleRef");
 		}
 
 		/// <summary>
@@ -525,6 +508,9 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type is static, false otherwise.</returns>
 		public static bool IsStatic (this TypeReference self)
 		{
+			if (self == null)
+				return false;
+
 			TypeDefinition type = self.Resolve ();
 			if (type == null)
 				return false;
@@ -538,6 +524,9 @@ namespace Gendarme.Framework.Rocks {
 		/// <returns>True if the type can be used from outside of the assembly, false otherwise.</returns>
 		public static bool IsVisible (this TypeReference self)
 		{
+			if (self == null)
+				return false;
+
 			TypeDefinition type = self.Resolve ();
 			if (type == null)
 				return true; // it's probably visible since we have a reference to it
