@@ -38,12 +38,23 @@ using Mono.Cecil;
 using Mono.Cecil.Cil;
 using NUnit.Framework;
 
+using Test.Rules.Definitions;
+using Test.Rules.Fixtures;
 using Test.Rules.Helpers;
 
 namespace Test.Rules.BadPractice {
 
 	[TestFixture]
-	public class CheckNewThreadWithoutStartTest {
+	public class CheckNewThreadWithoutStartTest : MethodRuleTestFixture<CheckNewThreadWithoutStartRule> {
+
+		[Test]
+		public void DoesNotApply ()
+		{
+			// no IL
+			AssertRuleDoesNotApply (SimpleMethods.ExternalMethod);
+			// no NEWOBJ
+			AssertRuleDoesNotApply (SimpleMethods.EmptyMethod);
+		}
 
 		private AssemblyDefinition assembly;
 		private TypeDefinition type;
@@ -54,8 +65,8 @@ namespace Test.Rules.BadPractice {
 		public void FixtureSetUp ()
 		{
 			string unit = Assembly.GetExecutingAssembly ().Location;
-			assembly = AssemblyFactory.GetAssembly (unit);
-			type = assembly.MainModule.Types ["Test.Rules.BadPractice.CheckNewThreadWithoutStartTest"];
+			assembly = AssemblyDefinition.ReadAssembly (unit);
+			type = assembly.MainModule.GetType ("Test.Rules.BadPractice.CheckNewThreadWithoutStartTest");
 			rule = new CheckNewThreadWithoutStartRule ();
 			runner = new TestRunner (rule);
 		}
