@@ -30,6 +30,7 @@ using System.Xml;
 using System.Collections.Generic;
 using NUnit.Framework;
 
+using Gendarme.Framework;
 using Gendarme.Rules.Correctness;
 
 using Test.Rules.Fixtures;
@@ -42,19 +43,14 @@ namespace Test.Rules.Correctness {
 		string foo;
 		StreamReader sr;
 
-		string DoesNotApply1 () { //no call/newobj/stloc
+		StreamReader StreamReader { get; set; }
+
+		string DoesNotApply1 ()
+		{
+			//no call/newobj/stloc
 			return foo;
 		}
 
-		StreamReader DoesNotApply2 () { //returns IDisposable
-			var sr = new StreamReader ("bar.xml");
-			return sr;
-		}
-
-		void DoesNotApply3 () {
-			sr = new StreamReader ("bar.xml"); //field
-		}
-
 		string Success0 () {
 			var o = new object ();
 			return o.ToString ();
@@ -119,6 +115,14 @@ namespace Test.Rules.Correctness {
 			yield return "bar";
 		}
 
+		void Success7 () {
+			sr = new StreamReader ("bar.xml"); //field
+		}
+
+		void Success8 () {
+			StreamReader = new StreamReader ("bar.xml"); //property
+		}
+
 		void Failure0 () {
 			var reader = XmlReader.Create ("foo.xml");
 			Console.WriteLine (reader.ToString ());
@@ -174,6 +178,20 @@ namespace Test.Rules.Correctness {
 			Success3 (reader);
 			reader = XmlReader.Create ("bar.xml");
 		}
+
+		void Failure7 () {
+			new XmlTextReader ("foo.xml");
+		}
+
+		void Failure8 () {
+			var xslt = new System.Xml.Xsl.XslCompiledTransform ();
+			xslt.Load (new XmlTextReader ("foo.xml"));
+		}
+
+		void Failure9 () {
+			var xslt = new System.Xml.Xsl.XslCompiledTransform ();
+			xslt.Load (XmlTextReader.Create ("foo.xml"));
+		}
 	}
 
 	[TestFixture]
@@ -183,6 +201,7 @@ namespace Test.Rules.Correctness {
 		public void DoesNotApply0 ()
 		{
 			AssertRuleDoesNotApply (SimpleMethods.EmptyMethod);
+			AssertRuleDoesNotApply (SimpleMethods.ExternalMethod);
 		}
 
 		[Test]
@@ -191,18 +210,6 @@ namespace Test.Rules.Correctness {
 			AssertRuleDoesNotApply<DisposalCases> ("DoesNotApply1");
 		}
 
-		[Test]
-		public void DoesNotApply2 ()
-		{
-			AssertRuleDoesNotApply<DisposalCases> ("DoesNotApply2");
-		}
-
-		[Test]
-		public void DoesNotApply3 ()
-		{
-			AssertRuleDoesNotApply<DisposalCases> ("DoesNotApply3");
-		}
-
 		[Test]
 		public void Success0 ()
 		{
@@ -245,6 +252,18 @@ namespace Test.Rules.Correctness {
 			AssertRuleSuccess<DisposalCases> ("Success6");
 		}
 
+		[Test]
+		public void Success7 ()
+		{
+			AssertRuleSuccess<DisposalCases> ("Success7");
+		}
+
+		[Test]
+		public void Success8 ()
+		{
+			AssertRuleSuccess<DisposalCases> ("Success8");
+		}
+
 		[Test]
 		public void ValueTypeDisposable ()
 		{
@@ -272,7 +291,7 @@ namespace Test.Rules.Correctness {
 		[Test]
 		public void Failure3 ()
 		{
-			AssertRuleFailure<DisposalCases> ("Failure3", 2);
+			AssertRuleFailure<DisposalCases> ("Failure3", 1);
 		}
 
 		[Test]
@@ -292,5 +311,133 @@ namespace Test.Rules.Correctness {
 		{
 			AssertRuleFailure<DisposalCases> ("Failure6", 2);
 		}
+
+		[Test]
+		public void Failure7 ()
+		{
+			AssertRuleFailure<DisposalCases> ("Failure7", 1);
+		}
+
+		[Test]
+		public void Failure8 ()
+		{
+			AssertRuleFailure<DisposalCases> ("Failure8", 1);
+		}
+
+		[Test]
+		public void Failure9 ()
+		{
+			AssertRuleFailure<DisposalCases> ("Failure9", 1);
+		}
+
+		// test case based on:
+		// https://github.com/Iristyle/mono-tools/commit/2cccfd0efd406434e1309d0740826ff06d32de20
+
+		string FluentTestCase ()
+		{
+			using (StringWriter sw = new StringWriter ()) {
+				// while analyzing 'FluentTestCase' we cannot know what's inside
+				// 'NestedFluentCall[Two|Three]' except that they _looks_like_ fluent APIs
+				return NestedFluentCall (sw).ToString () + 
+					NestedFluentCallTwo (sw).ToString () +
+					NestedFluentCallThree (sw).ToString ();
+			}
+		}
+
+		StringWriter NestedFluentCall (StringWriter stringWriter)
+		{
+			// same instance is returned and does not need to be disposed (the caller does it)
+			return stringWriter;
+		}
+
+		StringWriter NestedFluentCallTwo (StringWriter stringWriter)
+		{
+			// a new instance is being returned and someone should be disposing it
+			// without source code or (good) documentation it behaves exactly like
+			// the previous method
+			StringWriter sw = new StringWriter ();
+			sw.Write (stringWriter);
+			return sw;
+		}
+
+		StringWriter NestedFluentCallThree (StringWriter stringWriter)
+		{
+			// really bad code to show that we cannot determine with 100% exactitude
+			// what some methods returns to us
+			if (stringWriter.GetHashCode () % 2 == 1)
+				return NestedFluentCall (stringWriter);
+			else
+				return NestedFluentCallTwo (stringWriter);
+		}
+
+		[Test]
+		public void FluentApi ()
+		{
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("FluentTestCase");
+		}
+
+		// adapted (without locals variants) from https://bugzilla.novell.com/show_bug.cgi?id=666403
+
+		void OutParameter1 (out StreamReader stream)
+		{
+			var new_stream = new StreamReader ("baz.xml"); //out param
+			stream = new_stream;
+		}
+
+		bool OutParameter2 (out StreamReader stream)
+		{
+			stream = new StreamReader ("baz.xml"); //out param, no locals
+			return true;
+		}
+
+		[Test]
+		public void OutParameters ()
+		{
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("OutParameter1");
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("OutParameter2");
+		}
+
+		class SomeClassThatContainsADisposableProperty {
+			public StreamReader Reader { get; set; }
+		}
+
+		void OtherInstanceProperty1 (SomeClassThatContainsADisposableProperty someObj)
+		{
+			var reader = new StreamReader ("foobaz.xml");
+			someObj.Reader = reader; //property in param
+		}
+
+		void OtherInstanceProperty2 (SomeClassThatContainsADisposableProperty someObj)
+		{
+			someObj.Reader = new StreamReader ("foobaz.xml"); //property in param, no locals
+		}
+
+		[Test]
+		public void OtherInstance ()
+		{
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("OtherInstanceProperty1");
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("OtherInstanceProperty2");
+		}
+
+		StreamReader ReturnIDisposable1 ()
+		{
+			var ret = new StreamReader ("baz.xml"); //return value
+			return ret;
+		}
+
+		StreamReader ReturnIDisposable2 ()
+		{
+			return new StreamReader ("baz.xml"); //return value, no locals
+		}
+
+		[Test]
+		public void ReturnValue ()
+		{
+			// csc 10 (without /o optimize) will fail this as it introduce extra compiler generated locals
+#if __MonoCS__
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("ReturnIDisposable1");
+#endif
+			AssertRuleSuccess<EnsureLocalDisposalTest> ("ReturnIDisposable2");
+		}
 	}
 }
