@@ -28,6 +28,7 @@
 
 using System;
 using System.Collections.Generic;
+using System.Globalization;
 using Gendarme.Framework;
 using Gendarme.Framework.Rocks;
 using Gendarme.Framework.Helpers;
@@ -83,16 +84,19 @@ namespace Gendarme.Rules.Design {
 	public class DeclareEventHandlersCorrectlyRule : Rule, ITypeRule {
 		static IList<TypeReference> valid_event_handler_types = new List<TypeReference> ();
 
-		private bool CheckReturnVoid (TypeReference eventType, MethodReference invoke)
+		private bool CheckReturnVoid (IMetadataTokenProvider eventType, IMethodSignature invoke)
 		{
-			if (String.Compare (invoke.ReturnType.ReturnType.FullName, "System.Void") == 0)
+			TypeReference rtype = invoke.ReturnType;
+			if (rtype.IsNamed ("System", "Void"))
 				return true;
 
-			Runner.Report (eventType, Severity.Medium, Confidence.High, String.Format ("The delegate should return void, not {0}", invoke.ReturnType.ReturnType.FullName));
+			string msg = String.Format (CultureInfo.InvariantCulture, 
+				"The delegate should return void, not {0}", rtype.GetFullName ());
+			Runner.Report (eventType, Severity.Medium, Confidence.High, msg);
 			return false;
 		}
 
-		private bool CheckAmountOfParameters (TypeReference eventType, MethodReference invoke)
+		private bool CheckAmountOfParameters (IMetadataTokenProvider eventType, IMethodSignature invoke)
 		{
 			if (invoke.HasParameters && (invoke.Parameters.Count == 2))
 				return true;
@@ -101,18 +105,25 @@ namespace Gendarme.Rules.Design {
 			return false;
 		}
 
-		private bool CheckParameterTypes (TypeReference eventType, MethodReference invoke)
+		private bool CheckParameterTypes (IMetadataTokenProvider eventType, IMethodSignature invoke)
 		{
 			bool ok = true;
-			if (invoke.HasParameters && (invoke.Parameters.Count >= 1)) {
-				string type_name = invoke.Parameters [0].ParameterType.FullName;
-				if (String.Compare (type_name, "System.Object") != 0) {
-					Runner.Report (eventType, Severity.Medium, Confidence.High, String.Format ("The first parameter should have an object, not {0}", type_name));
+			if (!invoke.HasParameters)
+				return ok;
+
+			IList<ParameterDefinition> pdc = invoke.Parameters;
+			int count = pdc.Count;
+			if (count >= 1) {
+				TypeReference ptype = pdc [0].ParameterType;
+				if (!ptype.IsNamed ("System", "Object")) {
+					string msg = String.Format (CultureInfo.InvariantCulture, 
+						"The first parameter should have an object, not {0}", ptype.GetFullName ());
+					Runner.Report (eventType, Severity.Medium, Confidence.High, msg);
 					ok = false;
 				}
 			}
-			if (invoke.HasParameters && (invoke.Parameters.Count >= 2)) {
-				if (!invoke.Parameters [1].ParameterType.Inherits ("System.EventArgs")) {
+			if (count >= 2) {
+				if (!pdc [1].ParameterType.Inherits ("System", "EventArgs")) {
 					Runner.Report (eventType, Severity.Medium, Confidence.High, "The second parameter should be a subclass of System.EventArgs");
 					ok = false;
 				}
@@ -120,12 +131,14 @@ namespace Gendarme.Rules.Design {
 			return ok;
 		}
 
-		private bool CheckParameterName (TypeReference eventType, ParameterReference invokeParameter, string expectedName)
+		private bool CheckParameterName (IMetadataTokenProvider eventType, ParameterReference invokeParameter, string expectedName)
 		{
-			if (String.Compare (invokeParameter.Name, expectedName) == 0)
+			if (invokeParameter.Name == expectedName)
 				return true;
 
-			Runner.Report (eventType, Severity.Low, Confidence.High, String.Format ("The expected name is {0}, not {1}", expectedName, invokeParameter.Name));
+			string msg = String.Format (CultureInfo.InvariantCulture, "The expected name is {0}, not {1}", 
+				expectedName, invokeParameter.Name);
+			Runner.Report (eventType, Severity.Low, Confidence.High, msg);
 			return false;
 		}
 		
@@ -171,16 +184,20 @@ namespace Gendarme.Rules.Design {
 			bool valid = CheckReturnVoid (type, invoke);
 			valid &= CheckAmountOfParameters (type, invoke);
 			valid &= CheckParameterTypes (type, invoke);
-			if (invoke.Parameters.Count > 0)
-				valid &= CheckParameterName (type, invoke.Parameters [0], "sender");
-			if (invoke.Parameters.Count > 1)
-				valid &= CheckParameterName (type, invoke.Parameters [1], "e");
+
+			IList<ParameterDefinition> pdc = invoke.Parameters;
+			int count = pdc.Count;
+			if (count > 0) {
+				valid &= CheckParameterName (type, pdc [0], "sender");
+				if (count > 1)
+					valid &= CheckParameterName (type, pdc [1], "e");
+			}
 			return valid;
 		}
 
 		private bool CheckGenericDelegate (TypeReference type)
 		{
-			if (type.FullName == "System.EventHandler`1")
+			if (type.IsNamed ("System", "EventHandler`1"))
 				return true;
 
 			Runner.Report (type, Severity.Medium, Confidence.High, "Generic delegates should use EventHandler<TEventArgs>");
