@@ -34,8 +34,69 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
-	[Problem ("This type override the Equals method, or overloads + and - operators, but doesn't overload the == operator.")]
+	/// <summary>
+	/// This rule fires if a type overloads operator add <c>+</c>, or overloads operator subtract <c>-</c>,
+	/// or is a value type and overrides <c>Object.Equals</c>, but equals <c>==</c> is
+	/// not overloaded.
+	/// </summary>
+	/// <example>
+	/// Bad example (add/substract):
+	/// <code>
+	/// class DoesNotOverloadOperatorEquals {
+	///	public static int operator + (DoesNotOverloadOperatorEquals a)
+	///	{
+	///		return 0;
+	///	}
+	///	
+	///	public static int operator - (DoesNotOverloadOperatorEquals a)
+	///	{
+	///		return 0;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Bad example (value type):
+	/// <code>
+	/// struct OverridesEquals {
+	///	public override bool Equals (object obj)
+	///	{
+	///		return base.Equals (obj);
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// struct OverloadsOperatorEquals {
+	///	public static int operator + (OverloadsOperatorEquals a)
+	///	{
+	///		return 0;
+	///	}
+	///	
+	///	public static int operator - (OverloadsOperatorEquals a)
+	///	{
+	///		return 0;
+	///	}
+	///	
+	///	public static bool operator == (OverloadsOperatorEquals a, OverloadsOperatorEquals b)
+	///	{
+	///		return a.Equals (b);
+	///	}
+	///	
+	///	public override bool Equals (object obj)
+	///	{
+	///		return base.Equals (obj);
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
+	[Problem ("This type is a value type and overrides the Equals method or it overloads + and - operators but does not overload the == operator.")]
 	[Solution ("Overload the == operator to match the results of the Equals method.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1013:OverloadOperatorEqualsOnOverloadingAddAndSubtract")]
+	[FxCopCompatibility ("Microsoft.Usage", "CA2231:OverloadOperatorEqualsOnOverridingValueTypeEquals")]
 	public class OperatorEqualsShouldBeOverloadedRule : Rule, ITypeRule {
 
 		public RuleResult CheckType (TypeDefinition type)
