@@ -29,12 +29,16 @@
 //
 
 using System;
+using System.Collections.Generic;
+using System.Runtime.InteropServices;
 
 using Gendarme.Rules.Design;
+using Gendarme.Framework.Rocks;
 
 using NUnit.Framework;
 using Test.Rules.Definitions;
 using Test.Rules.Fixtures;
+using Test.Rules.Helpers;
 
 namespace Test.Rules.Design {
 
@@ -48,6 +52,12 @@ namespace Test.Rules.Design {
 		{
 			throw new NotImplementedException ();
 		}
+
+		IEnumerator<byte> GetBytes ()
+		{
+			for (int i = 0; i < 10; i++)
+				yield return Marshal.ReadByte (A, i);
+		}
 	}
 
 	class NoFinalizer : IDisposable {
@@ -115,6 +125,20 @@ namespace Test.Rules.Design {
 			AssertRuleDoesNotApply (SimpleTypes.Delegate);
 		}
 
+		[Test]
+		public void DoesNotApplyGeneratedCode ()
+		{
+			var declaring_type = DefinitionLoader.GetTypeDefinition<HasFinalizer> ();
+	
+			Assert.IsTrue (declaring_type.HasNestedTypes, "HasFinalizer-HasNestedTypes");
+			Assert.AreEqual (1, declaring_type.NestedTypes.Count, "HasFinalized-NestedTypesCount-1");
+
+			var type = declaring_type.NestedTypes [0];
+
+			Assert.IsTrue (type.IsGeneratedCode (), "NestedType-IsGeneratedCode-True");
+			AssertRuleDoesNotApply (type);
+		}
+
 		[Test]
 		public void TestHasFinalizer ()
 		{
