@@ -26,14 +26,15 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 // THE SOFTWARE.
 
+using Mono.Cecil;
 using Gendarme.Framework;
 
 namespace Gendarme.Rules.Exceptions {
 
 	/// <summary>
-	/// This rule check for methods that create basic exceptions like <c>System.Exception</c>,
+	/// This rule checks for methods that create basic exceptions like <c>System.Exception</c>,
 	/// <c>System.ApplicationException</c> or <c>System.SystemException</c>. Those exceptions
-	/// do not provide enough information about an error condition to be helpful to the consumer
+	/// do not provide enough information about the error to be helpful to the consumer
 	/// of the library.
 	/// </summary>
 	/// <example>
@@ -63,19 +64,16 @@ namespace Gendarme.Rules.Exceptions {
 	/// <remarks>This rule is available since Gendarme 2.0</remarks>
 
 	[Problem ("This method creates (and probably throws) an exception of Exception, ApplicationException or SystemException type.")]
-	[Solution ("Try to use more specific exception types. If none of existing types meet your needs, create custom exception class that inherits from System.Exception or any appropriate descendant of it.")]
+	[Solution ("Try to use a more specific exception type. If none of existing types meet your needs, create a custom exception class that inherits from System.Exception or any appropriate descendant of it.")]
 	[FxCopCompatibility ("Microsoft.Usage", "CA2201:DoNotRaiseReservedExceptionTypes")]
 	public class AvoidThrowingBasicExceptionsRule : NewExceptionsRule {
 
-		static string [] BasicExceptions = { 
-			"System.Exception", 
-			"System.ApplicationException", 
-			"System.SystemException"
-		};
-
-		protected override string [] GetExceptionTypes ()
+		protected override bool CheckException (TypeReference type)
 		{
-			return BasicExceptions;
+			if ((type == null) || (type.Namespace != "System"))
+				return false;
+			string name = type.Name;
+			return ((name == "Exception") || (name == "ApplicationException") || (name == "SystemException"));
 		}
 
 		protected override Severity Severity {
