@@ -5,43 +5,33 @@ using Mono.Cecil.Cil;
 
 namespace Gendarme.Rules.Exceptions.Impl {
 
-	public class ExecutionPathFactory {
-	
-		private MethodDefinition method;
+	internal sealed class ExecutionPathFactory {
 
-		public ExecutionPathFactory (MethodDefinition method)
+		const int MaxLimit = 1000;
+	
+		public ExecutionPathFactory ()
 		{
-			this.method = method;
 		}
 
 		public IList<ExecutionPathCollection> CreatePaths (Instruction start, Instruction end)
 		{
 			if (start == null)
 				throw new ArgumentNullException ("start");
-			if (end == null)
-				throw new ArgumentNullException ("end");
-#if DEBUG
-			if (!method.Body.Instructions.Contains (start))
-				throw new ArgumentException(
-					"start instruction is not contained in method " + 
-					method.DeclaringType.FullName + "::" + method.Name,
-					"start");
 
-			if (!method.Body.Instructions.Contains (end)) {
-				throw new ArgumentException(
-					"end instruction is not contained in method " + 
-					method.DeclaringType.FullName + "::" + method.Name,
-					"end");
-			}
-#endif
 			List<ExecutionPathCollection> paths = new List<ExecutionPathCollection> ();
 			CreatePathHelper (start, end, new ExecutionPathCollection (), paths);
+			// we were not able to build all code paths within a reasonable amount of memory
+			if (paths.Count >= MaxLimit)
+				paths.Clear ();
 			return paths;
 		}
 
 		private void CreatePathHelper (Instruction start, Instruction end, 
 			ExecutionPathCollection path, List<ExecutionPathCollection> completedPaths)
 		{
+			if (completedPaths.Count >= MaxLimit)
+				return;
+
 			ExecutionBlock curBlock = new ExecutionBlock ();
 			curBlock.First = start;
 
@@ -96,8 +86,6 @@ namespace Gendarme.Rules.Exceptions.Impl {
 					completedPaths.Add (path);
 					stop = true;
 					break;
-				default:
-					break;
 				}
 
 				if (cur.Next != null && cur != end && !stop)
