@@ -6,7 +6,7 @@
 //	Sebastien Pouliot <sebastien@ximian.com>
 //
 // (C) 2008 Néstor Salceda
-// Copyright (C) 2008 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2008,2010 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,6 +29,7 @@
 //
 
 using System;
+using System.Collections.Generic;
 
 using Mono.Cecil;
 using Mono.Cecil.Cil;
@@ -91,58 +92,83 @@ namespace Gendarme.Rules.Exceptions {
 			if (operand == null)
 				return false;
 
+			// for most getter and setter the property name is used
 			if (method.IsProperty ()) {
-				return String.Compare (method.Name, 4, operand, 0, operand.Length, StringComparison.Ordinal) == 0;
-			} else {
-				// note: we already know there are Parameters for this method is we got here
-				foreach (ParameterDefinition parameter in method.Parameters) {
-					if (parameter.Name == operand)
-						return true;
-				}
+				if (String.Compare (method.Name, 4, operand, 0, operand.Length, StringComparison.Ordinal) == 0)
+					return true;
+				// but we continue looking for parameters (e.g. indexers) unless it's the generic 'value' string
+				if (operand == "value")
+					return false;
+			}
+
+			// note: we already know there are Parameters for this method is we got here
+			foreach (ParameterDefinition parameter in method.Parameters) {
+				if (parameter.Name == operand)
+					return true;
 			}
 			return false;
 		}
 
-		private void CheckArgumentException (MethodReference ctor, Instruction ins, MethodDefinition method)
+		private void Report (MethodDefinition method, Instruction ins, string name)
+		{
+			Severity severity = ((name == "value") && method.IsProperty () &&
+				!method.Name.EndsWith ("value", StringComparison.InvariantCultureIgnoreCase)) ?
+				Severity.Low : Severity.High;
+
+			Runner.Report (method, ins, severity, Confidence.Normal);
+		}
+
+		private void CheckArgumentException (IMethodSignature ctor, Instruction ins, MethodDefinition method)
 		{
 			// OK		public ArgumentException ()
+			if (!ctor.HasParameters)
+				return;
+
 			// OK		public ArgumentException (string message)
-			if (!ctor.HasParameters || (ctor.Parameters.Count < 2))
+			IList<ParameterDefinition> pdc = ctor.Parameters;
+			if (pdc.Count < 2)
 				return;
 
 			// OK		public ArgumentException (string message, Exception innerException)
-			if (ctor.Parameters [1].ParameterType.FullName != "System.String")
+			if (!pdc [1].ParameterType.IsNamed ("System", "String"))
 				return;
 
 			// CHECK	public ArgumentException (string message, string paramName)
 			// CHECK	public ArgumentException (string message, string paramName, Exception innerException)
 			Instruction call = ins.TraceBack (method, -1);
-			if (MatchesAnyParameter (method, (call.Operand as string)))
+			string name = call.Operand as string;
+			if (MatchesAnyParameter (method, name))
 				return;
 
-			Runner.Report (method, ins, Severity.High, Confidence.Normal);
+			Report (method, ins, name);
 		}
 
 		// ctors are identical for ArgumentNullException, ArgumentOutOfRangeException and DuplicateWaitObjectException
-		private void CheckOtherExceptions (MethodReference constructor, Instruction ins, MethodDefinition method)
+		private void CheckOtherExceptions (IMethodSignature constructor, Instruction ins, MethodDefinition method)
 		{
 			// OK		public ArgumentNullException ()
 			if (!constructor.HasParameters)
 				return;
 
-			int parameters = constructor.Parameters.Count;
 			// OK		protected ArgumentNullException (SerializationInfo info, StreamingContext context)
 			// OK		public ArgumentNullException (string message, Exception innerException)
-			if ((parameters == 2) && (constructor.Parameters [1].ParameterType.FullName != "System.String"))
+			IList<ParameterDefinition> pdc = constructor.Parameters;
+			if ((pdc.Count == 2) && !pdc [1].ParameterType.IsNamed ("System", "String"))
 				return;
 
 			// CHECK	public ArgumentNullException (string paramName)
 			// CHECK	public ArgumentNullException (string paramName, string message)
 			Instruction call = ins.TraceBack (method, 0);
-			if (MatchesAnyParameter (method, (call.Operand as string)))
+			
+			// call will be null if there is branching logic in the selection of a message - just fon't check in this case
+			if (call == null)
 				return;
 
-			Runner.Report (method, ins, Severity.High, Confidence.Normal);
+			string name = call.Operand as string;
+			if (MatchesAnyParameter (method, name))
+				return;
+
+			Report (method, ins, name);
 		}
 
 		public RuleResult CheckMethod (MethodDefinition method)
@@ -166,18 +192,19 @@ namespace Gendarme.Rules.Exceptions {
 					continue;
 
 				MethodReference ctor = (current.Operand as MethodReference);
+				TypeReference type = ctor.DeclaringType;
+				if (type.Namespace != "System")
+					continue;
 
-				switch (ctor.DeclaringType.FullName) {
-				case "System.ArgumentException":
+				switch (type.Name) {
+				case "ArgumentException":
 					CheckArgumentException (ctor, current, method);
 					break;
-				case "System.ArgumentNullException":
-				case "System.ArgumentOutOfRangeException":
-				case "System.DuplicateWaitObjectException":
+				case "ArgumentNullException":
+				case "ArgumentOutOfRangeException":
+				case "DuplicateWaitObjectException":
 					CheckOtherExceptions (ctor, current, method);
 					break;
-				default:
-					continue;
 				}
 			}
 
