@@ -79,8 +79,8 @@ namespace Gendarme.Rules.Maintainability {
 
 			// we only want to run this on assemblies that use 2.0 or later
 			// since String.IsNullOrEmpty did not exist before that
-			Runner.AnalyzeAssembly += delegate (object o, RunnerEventArgs e) {
-				Active = (e.CurrentAssembly.Runtime >= TargetRuntime.NET_2_0);
+			Runner.AnalyzeModule += delegate (object o, RunnerEventArgs e) {
+				Active = (e.CurrentModule.Runtime >= TargetRuntime.Net_2_0);
 			};
 		}
 
@@ -116,7 +116,14 @@ namespace Gendarme.Rules.Maintainability {
 			case Code.Ldloc_S:
 				return (ins.Operand as VariableDefinition).Name;
 			default:
-				return (ins.Operand == null) ? String.Empty : ins.Operand.ToString ();
+				object o = ins.Operand;
+				MemberReference mr = (o as MemberReference);
+				if (mr != null)
+					return mr.GetFullName ();
+				else if (o != null)
+					return o.ToString ();
+				else
+					return String.Empty;
 			}
 		}
 
@@ -183,7 +190,8 @@ namespace Gendarme.Rules.Maintainability {
 				return RuleResult.DoesNotApply;
 
 			// is there any Call or Callvirt instructions in the method
-			if (!OpCodeBitmask.Calls.Intersect (OpCodeEngine.GetBitmask (method)))
+			OpCodeBitmask calls = OpCodeBitmask.Calls;
+			if (!calls.Intersect (OpCodeEngine.GetBitmask (method)))
 				return RuleResult.DoesNotApply;
 
 			// go!
@@ -191,15 +199,15 @@ namespace Gendarme.Rules.Maintainability {
 			// we look for a call to String.Length property (since it's much easier 
 			// than checking a string being compared to null)
 			foreach (Instruction current in method.Body.Instructions) {
-				if (!OpCodeBitmask.Calls.Get (current.OpCode.Code))
+				if (!calls.Get (current.OpCode.Code))
 					continue;
 
 				MethodReference mr = (current.Operand as MethodReference);
-				if ((mr.Name == "get_Length") && (mr.DeclaringType.FullName == "System.String")) {
+				if (mr.IsNamed ("System", "String", "get_Length")) {
 					// now that we found it we check that
 					// 1 - we previously did a check null on the same value (that we already know is a string)
 					Instruction branch = PreLengthCheck (method, current.Previous);
-					if ((branch == null) || (branch.OpCode.Code == Code.Ldc_I4_0))
+					if (branch == null)
 						continue;
 					// 2 - we compare the return value (length) with 0
 					if (PostLengthCheck (current.Next, branch)) {
