@@ -2,9 +2,10 @@
 // Unit Test for AvoidComplexMethods Rule.
 //
 // Authors:
-//      Cedric Vivier <cedricv@neonux.com>
+//	Sebastien Pouliot  <sebastien@ximian.com>
 //
-//      (C) 2008 Cedric Vivier
+// 	(C) 2008 Cedric Vivier
+// Copyright (C) 2010 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -27,19 +28,19 @@
 //
 
 using System;
-using System.Reflection;
 using System.Runtime.InteropServices;
 using System.Text;
 
 using Gendarme.Framework;
+using Gendarme.Framework.Rocks;
 using Gendarme.Rules.Maintainability;
 using Mono.Cecil;
+using Mono.Cecil.Cil;
 
 using NUnit.Framework;
 using Test.Rules.Fixtures;
 using Test.Rules.Helpers;
 
-
 namespace Test.Rules.Maintainability {
 
 	[AttributeUsage(AttributeTargets.Method)]
@@ -308,6 +309,82 @@ namespace Test.Rules.Maintainability {
 			throw new InvalidOperationException ();
 		}
 
+		// adapted from InstructionRocks.GetOperandType
+		[ExpectedCC (8)]
+		public static int GetOperandType (Instruction self, MethodDefinition method)
+		{
+			int i = 0;
+			switch (self.OpCode.Code) {
+			case Code.Ldarg_0:
+			case Code.Ldarg_1:
+			case Code.Ldarg_2:
+			case Code.Ldarg_3:
+			case Code.Ldarg:
+			case Code.Ldarg_S:
+			case Code.Ldarga:
+			case Code.Ldarga_S:
+			case Code.Starg:
+			case Code.Starg_S:
+				i = 1;
+				Console.WriteLine ("arguments");
+				break;
+			case Code.Conv_R4:
+			case Code.Ldc_R4:
+			case Code.Ldelem_R4:
+			case Code.Ldind_R4:
+			case Code.Stelem_R4:
+			case Code.Stind_R4:
+				i = 2;
+				Console.WriteLine ("singles");
+				break;
+			case Code.Conv_R8:
+			case Code.Ldc_R8:
+			case Code.Ldelem_R8:
+			case Code.Ldind_R8:
+			case Code.Stelem_R8:
+				i = 3;
+				Console.WriteLine ("doubles");
+				break;
+			case Code.Ldloc_0:
+			case Code.Ldloc_1:
+			case Code.Ldloc_2:
+			case Code.Ldloc_3:
+			case Code.Ldloc:
+			case Code.Ldloc_S:
+			case Code.Ldloca:
+			case Code.Ldloca_S:
+			case Code.Stloc_0:
+			case Code.Stloc_1:
+			case Code.Stloc_2:
+			case Code.Stloc_3:
+			case Code.Stloc:
+			case Code.Stloc_S:
+				i = 4;
+				Console.WriteLine ("locals");
+				break;
+			case Code.Ldfld:
+			case Code.Ldflda:
+			case Code.Ldsfld:
+			case Code.Ldsflda:
+			case Code.Stfld:
+			case Code.Stsfld:
+				i = 5;
+				Console.WriteLine ("fields");
+				break;
+			case Code.Call:
+			case Code.Callvirt:
+			case Code.Newobj:
+				i = 6;
+				Console.WriteLine ("calls");
+				break;
+			default:
+				i = 7;
+				Console.WriteLine ("default");
+				break;
+			}
+			Console.WriteLine ("end");
+			return i;
+		}
 	}
 
 
@@ -323,9 +400,12 @@ namespace Test.Rules.Maintainability {
 			int expected_cc;
 
 			foreach (MethodDefinition method in type.Methods) {
-				actual_cc = Rule.GetCyclomaticComplexityForMethod (method);
+				if (method.IsConstructor)
+					continue;
+
+				actual_cc = AvoidComplexMethodsRule.GetCyclomaticComplexity (method);
 				expected_cc = GetExpectedComplexity (method);
-				Assert.AreEqual (actual_cc, expected_cc,
+				Assert.AreEqual (expected_cc, actual_cc,
 					"CC for method '{0}' is {1} but should have been {2}.",
 					method.Name, actual_cc, expected_cc);
 			}
@@ -334,8 +414,8 @@ namespace Test.Rules.Maintainability {
 		private int GetExpectedComplexity(MethodDefinition method)
 		{
 			foreach (CustomAttribute attr in method.CustomAttributes) {
-				if (attr.Constructor.DeclaringType.Name == "ExpectedCCAttribute")
-					return (int) attr.ConstructorParameters [0];
+				if (attr.AttributeType.Name == "ExpectedCCAttribute")
+					return (int) attr.ConstructorArguments [0].Value;
 			}
 			
 			throw new ArgumentException (method + " does not have ExpectedCCAttribute");
@@ -394,5 +474,11 @@ namespace Test.Rules.Maintainability {
 			}
 		}
 
+		[Test]
+		public void HighLevelSwitchBrokenIntoSeveralIlSwitch ()
+		{
+			AssertRuleSuccess<MethodsWithExpectedCC> ("GetOperandType");
+		}
 	}
 }
+
