@@ -65,7 +65,7 @@ namespace Test.Rules.Naming {
 		public void FixtureSetUp ()
 		{
 			string unit = Assembly.GetExecutingAssembly ().Location;
-			assembly = AssemblyFactory.GetAssembly (unit);
+			assembly = AssemblyDefinition.ReadAssembly (unit);
 			rule = new UsePluralNameInEnumFlagsRule ();
 			runner = new TestRunner (rule);
 		}
@@ -73,7 +73,7 @@ namespace Test.Rules.Naming {
 		[Test]
 		public void TestFlagsHasPluralName () 
 		{
-			type = assembly.MainModule.Types ["Test.Rules.Naming.AppDomainManagerInitializationOptions"];
+			type = assembly.MainModule.GetType ("Test.Rules.Naming.AppDomainManagerInitializationOptions");
 			Assert.AreEqual (RuleResult.Success, runner.CheckType (type), "RuleResult");
 			Assert.AreEqual (0, runner.Defects.Count, "Count");
 		}
@@ -81,7 +81,7 @@ namespace Test.Rules.Naming {
 		[Test]
 		public void TestFlagsHasSingularName () 
 		{
-			type = assembly.MainModule.Types ["Test.Rules.Naming.ConsoleModifier"];
+			type = assembly.MainModule.GetType ("Test.Rules.Naming.ConsoleModifier");
 			Assert.AreEqual (RuleResult.Failure, runner.CheckType (type), "RuleResult");
 			Assert.AreEqual (1, runner.Defects.Count, "Count");
 		}
