@@ -6,7 +6,7 @@
 //	Sebastien Pouliot  <sebastien@ximian.com>
 //
 //  (C) 2007 Daniel Abramov
-// Copyright (C) 2008 Novell, Inc (http://www.novell.com)
+// Copyright (C) 2008, 2010 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -29,6 +29,7 @@
 //
 
 using System;
+using System.Globalization;
 
 using Mono.Cecil;
 using Gendarme.Framework;
@@ -87,7 +88,14 @@ namespace Gendarme.Rules.Naming {
 		{
 			if (name.Length < 3)
 				return true;
-			return !((name [0] == 'C') && Char.IsUpper (name [1]) && Char.IsLower (name [2]));
+
+			switch (name [0]) {
+			case 'C':	// MFC like CMyClass should fail - but works for CLSCompliant
+			case 'I':	// interface-like - Classes beginning with In or Is etc should pass, e.g. InMemoryDoohicky
+				return Char.IsLower (name [1]) ? true : Char.IsUpper (name [2]);
+			default:
+				return true;
+			}
 		}
 
 		private static bool IsCorrectInterfaceName (string name)
@@ -107,16 +115,21 @@ namespace Gendarme.Rules.Naming {
 			if (type.IsGeneratedCode ())
 				return RuleResult.DoesNotApply;
 
+			string name = type.Name;
 			if (type.IsInterface) {
 				// interfaces should look like 'ISomething'
-				if (!IsCorrectInterfaceName (type.Name)) { 
-					string s = String.Format ("The '{0}' interface name doesn't have the required 'I' prefix. Acoording to existing naming conventions, all interface names should begin with the 'I' letter followed by another capital letter.", type.Name);
+				if (!IsCorrectInterfaceName (name)) { 
+					string s = String.Format (CultureInfo.InvariantCulture,
+						"The '{0}' interface name doesn't have the required 'I' prefix. Acoording to existing naming conventions, all interface names should begin with the 'I' letter followed by another capital letter.", 
+						name);
 					Runner.Report (type, Severity.Critical, Confidence.High, s);
 				}
 			} else {
-				// class should _not_ look like 'CSomething"
-				if (!IsCorrectTypeName (type.Name)) { 
-					string s = String.Format ("The '{0}' type name starts with 'C' prefix but, according to existing naming conventions, type names should not have any specific prefix.", type.Name);
+				// class should _not_ look like 'CSomething" or like an interface 'IOops'
+				if (!IsCorrectTypeName (name)) { 
+					string s = String.Format (CultureInfo.InvariantCulture,
+						"The '{0}' type name starts with '{1}' prefix but, according to existing naming conventions, type names should not have any specific prefix.", 
+						name, name [0]);
 					Runner.Report (type, Severity.Medium, Confidence.High, s);
 				}
 			}
@@ -126,8 +139,11 @@ namespace Gendarme.Rules.Naming {
 				// but if they are longer (than one char) they should start with a 'T'
 				// e.g. EventHandler<TEventArgs>
 				foreach (GenericParameter parameter in type.GenericParameters) {
-					if (IsCorrectGenericParameterName (parameter.Name)) {
-						string s = String.Format ("The generic parameter '{0}' should be prefixed with 'T' or be a single, uppercased letter.", parameter.Name);
+					string param_name = parameter.Name;
+					if (IsCorrectGenericParameterName (param_name)) {
+						string s = String.Format (CultureInfo.InvariantCulture,
+							"The generic parameter '{0}' should be prefixed with 'T' or be a single, uppercased letter.", 
+							param_name);
 						Runner.Report (type, Severity.High, Confidence.High, s);
 					}
 				}
