@@ -32,9 +32,46 @@ using Gendarme.Framework;
 using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Performance {
-	
-	[Problem ("Due performance issues, concrete attributes should be sealed.")]
-	[Solution ("Unless you plan to inherit from this attribute you should consider to seal it's type.")]
+
+	/// <summary>
+	/// This rule fires if an attribute is defined which is both concrete (i.e. not abstract)
+	/// and unsealed. This is a performance problem because it means that 
+	/// <c>System.Attribute.GetCustomAttribute</c> has to search the attribute type
+	/// hierarchy for derived types. To fix this either seal the type or make it abstract. 
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// [AttributeUsage (AttributeTargets.All)]
+	/// public class BadAttribute : Attribute {
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (sealed):
+	/// <code>
+	/// [AttributeUsage (AttributeTargets.All)]
+	/// public sealed class SealedAttribute : Attribute {
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (abstract and sealed):
+	/// <code>
+	/// [AttributeUsage (AttributeTargets.All)]
+	/// public abstract class AbstractAttribute : Attribute {
+	/// }
+	/// 
+	/// [AttributeUsage (AttributeTargets.All)]
+	/// public sealed class ConcreteAttribute : AbstractAttribute {
+	/// }
+	/// </code>
+	/// </example>
+	/// <remarks>Before Gendarme 2.0 this rule was named AvoidUnsealedAttributesRule.</remarks>
+
+	[Problem ("Because of performance issues, concrete attributes should be sealed.")]
+	[Solution ("Unless you plan to inherit from this attribute you should consider sealing it.")]
+	[FxCopCompatibility ("Microsoft.Performance", "CA1813:AvoidUnsealedAttributes")]
 	public class AvoidUnsealedConcreteAttributesRule : Rule, ITypeRule {
 
 		public RuleResult CheckType (TypeDefinition type)
@@ -49,7 +86,7 @@ namespace Gendarme.Rules.Performance {
 			if (type.IsSealed) // it's ok
 				return RuleResult.Success;
 
-			Runner.Report (type, Severity.Medium, Confidence.High, "Due to performance issues, attributes should all be declared sealed");
+			Runner.Report (type, Severity.Medium, Confidence.High);
 			return RuleResult.Failure;
 		}
 	}
