@@ -3,8 +3,10 @@
 //
 // Authors:
 //	Daniel Abramov <ex@vingrad.ru>
+//	Sebastien Pouliot <sebastien@ximian.com>
 //
 // Copyright (C) Daniel Abramov
+// Copyright (C) 2010 Novell, Inc (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -25,14 +27,12 @@
 // THE SOFTWARE.
 
 using System;
+using System.Diagnostics.CodeAnalysis;
 
-using Mono.Cecil;
-
-using Gendarme.Framework;
 using Gendarme.Rules.Performance;
 
 using NUnit.Framework;
-using Test.Rules.Helpers;
+using Test.Rules.Fixtures;
 
 namespace Test.Rules.Performance {
 
@@ -52,56 +52,75 @@ namespace Test.Rules.Performance {
 	}
 
 	[TestFixture]
-	public class AvoidUnsealedConcreteAttributesTest {
-
-		private ITypeRule rule;
-		private AssemblyDefinition assembly;
-		private TestRunner runner;
-
+	public class AvoidUnsealedConcreteAttributesTest : TypeRuleTestFixture<AvoidUnsealedConcreteAttributesRule> {
 
 		[TestFixtureSetUp]
-		public void FixtureSetUp ()
+		public void SetUp ()
 		{
-			string unit = System.Reflection.Assembly.GetExecutingAssembly ().Location;
-			assembly = AssemblyFactory.GetAssembly (unit);
-			rule = new AvoidUnsealedConcreteAttributesRule ();
-			runner = new TestRunner (rule);
+			Runner.Engines.Subscribe ("Gendarme.Framework.Engines.SuppressMessageEngine");
 		}
-
-		private TypeDefinition GetTest<T> ()
-		{
-			return assembly.MainModule.Types [typeof (T).FullName];
-		}
-
+		
 		[Test]
 		public void TestAbstractAttribute ()
 		{
-			Assert.AreEqual (RuleResult.Success, runner.CheckType (GetTest<AbstractAttribute> ()));
+			AssertRuleSuccess<AbstractAttribute> ();
 		}
 
 		[Test]
 		public void TestAnAttribute ()
 		{
-			Assert.AreEqual (RuleResult.Failure, runner.CheckType (GetTest<AnAttribute> ()));
-			Assert.AreEqual (1, runner.Defects.Count);
+			AssertRuleFailure<AnAttribute> (1);
 		}
 
 		[Test]
 		public void TestNotAttribute ()
 		{
-			Assert.AreEqual (RuleResult.DoesNotApply, runner.CheckType (GetTest<NotAttribute> ()));
+			AssertRuleDoesNotApply<NotAttribute> ();
 		}
 
 		[Test]
 		public void TestSealedAttribute ()
 		{
-			Assert.AreEqual (RuleResult.Success, runner.CheckType (GetTest<SealedAttribute> ()));
+			AssertRuleSuccess<SealedAttribute> ();
 		}
 
 		[Test]
 		public void TestSealedAttributeInheritsAnAttribute ()
 		{
-			Assert.AreEqual (RuleResult.Success, runner.CheckType (GetTest<SealedAttributeInheritsAnAttribute> ()));
+			AssertRuleSuccess<SealedAttributeInheritsAnAttribute> ();
+		}
+
+		public class FxCopTest {
+
+			// CA1813
+			public class AvoidUnsealedAttributes {
+
+				public class Fail : Attribute {
+				}
+
+				// manually suppressed - no MessageId
+				[SuppressMessage ("Microsoft.Performance", "CA1813:AvoidUnsealedAttributes")]
+				public class ManuallySuppressed : Attribute {
+				}
+
+				// automatically suppressed using VS2010
+				[System.Diagnostics.CodeAnalysis.SuppressMessage ("Microsoft.Performance", "CA1813:AvoidUnsealedAttributes")]
+				public class AutomaticallySuppressed : Attribute {
+				}
+
+				// automatically suppressed using VS2010 (see GlobalSupressions.cs)
+				public class GloballySuppressed : Attribute {
+				}
+			}
+		}
+
+		[Test]
+		public void CA1813 ()
+		{
+			AssertRuleFailure<FxCopTest.AvoidUnsealedAttributes.Fail> (1);
+			AssertRuleDoesNotApply<FxCopTest.AvoidUnsealedAttributes.ManuallySuppressed> ();
+			AssertRuleDoesNotApply<FxCopTest.AvoidUnsealedAttributes.AutomaticallySuppressed> ();
+			AssertRuleDoesNotApply<FxCopTest.AvoidUnsealedAttributes.GloballySuppressed> ();
 		}
 	}
 }
