@@ -107,7 +107,9 @@ namespace Gendarme.Rules.Portability {
 			// but we want to avoid checking all methods if the Environment type
 			// isn't referenced in a module (big performance difference)
 			Runner.AnalyzeModule += delegate (object o, RunnerEventArgs e) {
-				Active = e.CurrentAssembly.MainModule.TypeReferences.ContainsType ("System.Environment");
+				Active = e.CurrentModule.AnyTypeReference ((TypeReference tr) => {
+					return tr.IsNamed ("System", "Environment");
+				});
 			};
 		}
 
@@ -138,9 +140,10 @@ namespace Gendarme.Rules.Portability {
 				return RuleResult.DoesNotApply;
 
 			// the rule does not apply of the entry point returns void
-			// FIXME: entryPoint.ReturnType.ReturnType should not be null with void Main ()
+			// FIXME: entryPoint.ReturnType should not be null with void Main ()
 			// either bad unit tests or bug in cecil
-			if (entry_point.ReturnType.ReturnType == null || entry_point.ReturnType.ReturnType.FullName != "System.Int32")
+			TypeReference rt = entry_point.ReturnType;
+			if (!rt.IsNamed ("System", "Int32"))
 				return RuleResult.DoesNotApply;
 
 			Instruction previous = null;
@@ -187,7 +190,7 @@ namespace Gendarme.Rules.Portability {
 				return (a >= 0 && a <= 255) ? InspectionResult.Good : InspectionResult.Bad;
 			case Code.Call:
 			case Code.Callvirt:
-				if ((instruction.Operand as MethodReference).ReturnType.ReturnType.FullName == "System.Byte")
+				if ((instruction.Operand as MethodReference).ReturnType.IsNamed ("System", "Byte"))
 					return InspectionResult.Good;
 				else
 					return InspectionResult.Unsure; // could be within 0-255 or not
@@ -213,9 +216,10 @@ namespace Gendarme.Rules.Portability {
 				case Code.Call:
 				case Code.Callvirt:
 					MethodReference calledMethod = (MethodReference) current.Operand;
-					if (calledMethod.Name != "set_ExitCode" && calledMethod.Name != "Exit")
+					string name = calledMethod.Name;
+					if ((name != "set_ExitCode") && (name != "Exit"))
 						break;
-					if (calledMethod.DeclaringType.FullName != "System.Environment")
+					if (!calledMethod.DeclaringType.IsNamed ("System", "Environment"))
 						break;
 
 					InspectionResult result = CheckInstruction (previous);
