@@ -35,15 +35,63 @@ using Mono.Cecil;
 
 namespace Gendarme.Rules.UI {
 
-	[Problem ("The System.Windows.Forms applications entry-point (Main) is missing an [STAThread] attribute.")]
-	[Solution ("Add a [STAThread] attribute to your application Main method.")]
+	/// <summary>
+	/// This rule checks executable assemblies, i.e. *.exe's, that reference 
+	/// System.Windows.Forms to 
+	/// ensure that their entry point is decorated with <c>[System.STAThread]</c> attribute 
+	/// and is not decorated with <c>[System.MTAThread]</c> attribute to ensure that Windows 
+	/// Forms work properly.
+	/// </summary>
+	/// <example>
+	/// Bad example #1 (no attributes):
+	/// <code>
+	/// public class WindowsFormsEntryPoint {
+	///	static void Main ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Bad example #2 (MTAThread)
+	/// <code>
+	/// public class WindowsFormsEntryPoint {
+	///	[MTAThread]
+	///	static void Main ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example #1 (STAThread):
+	/// <code>
+	/// public class WindowsFormsEntryPoint {
+	///     [STAThread]
+	///     static void Main ()
+	///     {
+	///     }
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example #2 (not Windows Forms):
+	/// <code>
+	/// public class ConsoleAppEntryPoint {
+	///	static void Main ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
+	[Problem ("The System.Windows.Forms application's entry-point (Main) is missing an [STAThread] attribute.")]
+	[Solution ("Add a [STAThread] attribute to your application's Main method.")]
+	[FxCopCompatibility ("Microsoft.Usage", "CA2232:MarkWindowsFormsEntryPointsWithStaThread")]
 	public class UseSTAThreadAttributeOnSWFEntryPointsRule : Rule, IAssemblyRule {
 
 		private const string SystemWindowsForms = "System.Windows.Forms";
 
-		private const string STAThread = "System.STAThreadAttribute";
-		private const string MTAThread = "System.MTAThreadAttribute";
-
 		public RuleResult CheckAssembly (AssemblyDefinition assembly)
 		{
 			MethodDefinition entry_point = assembly.EntryPoint;
@@ -64,8 +112,8 @@ namespace Gendarme.Rules.UI {
 			if (!referencesSWF)
 				return RuleResult.DoesNotApply;
 
-			bool hasSTA = entry_point.HasAttribute (STAThread);
-			bool hasMTA = entry_point.HasAttribute (MTAThread);
+			bool hasSTA = entry_point.HasAttribute ("System", "STAThreadAttribute");
+			bool hasMTA = entry_point.HasAttribute ("System", "MTAThreadAttribute");
 
 			// success if only [STAThread] attribute is present
 			if (hasSTA && !hasMTA)
