@@ -143,21 +143,61 @@ namespace GuiCompare
 
 		string [] api_sl2 = {
 			"mscorlib",
-			"System.Windows",
-			"Microsoft.VisualBasic",
 			"System",
 			"System.Core",
 			"System.Net",
 			"System.Runtime.Serialization",
 			"System.ServiceModel",
+			"System.ServiceModel.Web",
+			"System.Windows",
 			"System.Windows.Browser",
 			"System.Xml",
+			"Microsoft.VisualBasic",
 			"",
 			// sdk assemblies:
 			"System.Xml.Linq",
 			"System.Windows.Controls",
 			"System.Windows.Controls.Data",
 		};
+		
+		string [] api_sl4 = {
+			"mscorlib",
+			"System",
+			"System.Core",
+			"System.Net",
+			"System.Runtime.Serialization",
+			"System.ServiceModel",
+			"System.ServiceModel.Web",
+			"System.Windows",
+			"System.Windows.Browser",
+			"System.Xml",
+			"Microsoft.VisualBasic",
+			"",
+			// sdk assemblies:
+			"Microsoft.CSharp",
+			"System.ComponentModel.Composition",
+			"System.ComponentModel.Composition.Initialization",
+			"System.ComponentModel.DataAnnotations",
+			"System.Data.Services.Client",
+			"System.Json",
+			"System.Numerics",
+			"System.Runtime.Serialization.Json",
+			"System.ServiceModel.Extensions",
+			"System.ServiceModel.NetTcp",
+			"System.ServiceModel.PollingDuplex",
+			"System.ServiceModel.Syndication",
+			"System.ServiceModel.Web.Extensions",
+			"System.Windows.Controls.Data",
+			"System.Windows.Controls.Data.Input",
+			"System.Windows.Controls",
+			"System.Windows.Controls.Input",
+			"System.Windows.Controls.Navigation",
+			"System.Windows.Data",
+			"System.Xml.Linq",
+			"System.Xml.Serialization",
+			"System.Xml.Utils",
+			"System.Xml.XPath"
+		};
 
 		string [] api_3_5 = {
 			"mscorlib",
@@ -165,6 +205,7 @@ namespace GuiCompare
 			"System.AddIn",
 			"System.AddIn.Contract",
 			"System.Configuration",
+			"System.ComponentModel.DataAnnotations",
 			"System.Core", 
 			// "System.Configuration.Install",
 			"System.Data",
@@ -198,6 +239,8 @@ namespace GuiCompare
 			"Microsoft.Build.Framework",
 			"Microsoft.Build.Tasks",
 			"Microsoft.Build.Utilities",
+			"Microsoft.Build.Conversion.v3.5",
+			"Microsoft.Build.Utilities.v3.5",
 			"",
 			"System.Configuration.Install",
 			"System.Design",
@@ -210,15 +253,13 @@ namespace GuiCompare
 
 		string [] api_4_0 = {
 			"mscorlib",
-			
-			"System.Activities.Core.Design",
-			"System.Activities.Design.Base",
+
+			"System.Activities",			
+			"System.Activities.Core.Presentation",
+			"System.Activities.DurableInstancing",
 			"System.Activities.Design",
-			"System.Activities",
-			"System.Activities.Extended",
 			"System.AddIn.Contract",
 			"System.AddIn",
-			"System.Caching",
 			"System.ComponentModel.Composition",
 			"System.ComponentModel.DataAnnotations",
 			"System.configuration",
@@ -236,6 +277,7 @@ namespace GuiCompare
 			"System.Data.SqlXml",
 			"System.Deployment",
 			"System.Design",
+			"System.Device",
 		//	"System.DirectoryServices.AccountManagement",
 			"System.DirectoryServices",
 		//	"System.DirectoryServices.Protocols",
@@ -253,12 +295,14 @@ namespace GuiCompare
 		//	"System.Management.Instrumentation",
 			"System.Messaging",
 			"System.Net",
+			"System.Numerics",
 			"System.Printing",
-			"System.Runtime",
+			"System.Runtime.Caching",
 			"System.Runtime.Remoting",
 			"System.Runtime.Serialization",
 			"System.Runtime.Serialization.Formatters.Soap",
 			"System.Security",
+			"System.ServiceModel.Activation",
 			"System.ServiceModel.Activities",
 			"System.ServiceModel.Channels",
 			"System.ServiceModel.Discovery",
@@ -269,6 +313,7 @@ namespace GuiCompare
 			"System.Speech",
 			"System.Transactions",
 			"System.Web.Abstractions",
+			"System.Web.ApplicationServices",
 			"System.Web.DataVisualization.Design",
 			"System.Web.DataVisualization",
 			"System.Web",
@@ -314,19 +359,138 @@ namespace GuiCompare
 			"PresentationFramework",
 			"PresentationFramework.Luna",
 			"PresentationFramework.Royale",
-
+			"PresentationUI",
 			"ReachFramework",
-			"ReferenceAssemblyBuildTask",
 
 			"WindowsBase",
 		//	"XamlBuildTask"
 		};
-		
-		const string masterinfos_version = "2.6";
+
+		string [] api_4_5 = {
+			"mscorlib",
+
+			"System.Activities",			
+			"System.Activities.Core.Presentation",
+			"System.Activities.DurableInstancing",
+			"System.Activities.Design",
+			"System.AddIn.Contract",
+			"System.AddIn",
+			"System.ComponentModel.Composition",
+			"System.ComponentModel.DataAnnotations",
+			"System.configuration",
+		//	"System.Configuration.Install",
+			"System.Core",
+			"System.Data.DataSetExtensions",
+			"System.Data",
+			"System.Data.Entity.Design",
+			"System.Data.Entity",
+			"System.Data.Linq",
+			"System.Data.OracleClient",
+			"System.Data.Services.Client",
+			"System.Data.Services.Design",
+			"System.Data.Services",
+			"System.Data.SqlXml",
+			"System.Deployment",
+			"System.Design",
+			"System.Device",
+		//	"System.DirectoryServices.AccountManagement",
+			"System.DirectoryServices",
+		//	"System.DirectoryServices.Protocols",
+			"System",
+			"System.Drawing.Design",
+			"System.Drawing",
+			"System.Dynamic",
+			"System.EnterpriseServices",
+			"System.EnterpriseServices.Thunk",
+			"System.EnterpriseServices.Wrapper",
+			"System.IdentityModel",
+			"System.IdentityModel.Selectors",
+			"System.IO.Log",
+			"System.IO.Compression",
+			"System.IO.Compression.FileSystem",
+			"System.Management",
+		//	"System.Management.Instrumentation",
+			"System.Messaging",
+			"System.Net",
+			"System.Net.Http",
+			"System.Net.Http.WebRequest",
+			"System.Numerics",
+			"System.Printing",
+			"System.Runtime.Caching",
+			"System.Runtime.Remoting",
+			"System.Runtime.Serialization",
+			"System.Runtime.Serialization.Formatters.Soap",
+			"System.Security",
+			"System.ServiceModel.Activation",
+			"System.ServiceModel.Activities",
+			"System.ServiceModel.Channels",
+			"System.ServiceModel.Discovery",
+			"System.ServiceModel",
+			"System.ServiceModel.Routing",
+			"System.ServiceModel.Web",
+			"System.ServiceProcess",
+			"System.Speech",
+			"System.Threading.Tasks.Dataflow",
+			"System.Transactions",
+			"System.Web.Abstractions",
+			"System.Web.ApplicationServices",
+			"System.Web.DataVisualization.Design",
+			"System.Web.DataVisualization",
+			"System.Web",
+			"System.Web.DynamicData.Design",
+			"System.Web.DynamicData",
+			"System.Web.Entity.Design",
+			"System.Web.Entity",
+		//	"System.Web.Extensions.Design",
+			"System.Web.Extensions",
+		//	"System.Web.Mobile",
+		//	"System.Web.RegularExpressions",
+			"System.Web.Routing",
+			"System.Web.Services",
+			"System.Windows.Forms.DataVisualization.Design",
+			"System.Windows.Forms.DataVisualization",
+			"System.Windows.Forms",
+			"System.Windows.Presentation",
+			"System.Workflow.Activities",
+			"System.Workflow.ComponentModel",
+			"System.Workflow.Runtime",
+			"System.WorkflowServices",
+			"System.Xaml",
+			"System.Xaml.Hosting",
+			"System.Xml",
+			"System.Xml.Linq",
+
+			"Microsoft.Build.Conversion.v4.0",
+			"Microsoft.Build",
+			"Microsoft.Build.Engine",
+			"Microsoft.Build.Framework",
+			"Microsoft.Build.Tasks.v4.0",
+			"Microsoft.Build.Utilities.v4.0",
+			"Microsoft.CSharp",
+			"Microsoft.JScript",
+			"Microsoft.VisualBasic.Compatibility.Data",
+			"Microsoft.VisualBasic.Compatibility",
+			"Microsoft.VisualBasic",
+
+			"PresentationBuildTasks",
+			"PresentationCore",
+			"PresentationFramework.Aero",
+			"PresentationFramework.Classic",
+			"PresentationFramework",
+			"PresentationFramework.Luna",
+			"PresentationFramework.Royale",
+			"PresentationUI",
+			"ReachFramework",
+
+			"WindowsBase",
+		//	"XamlBuildTask"
+		};
+
+		const string masterinfos_version = "2.8";
 
 		static Uri GetMasterInfoUri (string file)
 		{
-			return new Uri (string.Format ("http://mono.ximian.com/masterinfos/{0}/{1}", masterinfos_version, file));
+			return new Uri (string.Format ("http://go-mono.com/masterinfos/{0}/{1}", masterinfos_version, file));
 		}
 		
 		public static void Init ()
@@ -374,9 +538,13 @@ namespace GuiCompare
 				break;
 				
 			case "4.0":
-				u = GetMasterInfoUri ("masterinfos-4.0.tar.gz");			
+				u = GetMasterInfoUri ("masterinfos-4.0.tar.gz");
 				break;
-			
+
+			case "4.5":
+				u = GetMasterInfoUri ("masterinfos-4.5.tar.gz");
+				break;
+
 			case "SL2":
 				u = GetMasterInfoUri ("masterinfos-SL2.tar.gz");
 				break;
@@ -385,6 +553,9 @@ namespace GuiCompare
 				u = GetMasterInfoUri ("masterinfos-SL3.tar.gz");
 				break;
 			
+			case "SL4":
+				u = GetMasterInfoUri ("masterinfos-SL4.tar.gz");
+				break;
 			default:
 				main.Status = "Profile is unknown";
 				return;
@@ -429,10 +600,11 @@ namespace GuiCompare
 						ProcessStartInfo pi = new ProcessStartInfo();
 						pi.WorkingDirectory = pdir;
 						pi.UseShellExecute = true;
-						pi.FileName = "tar xzf " + target + " --strip-components=1";
+						pi.FileName = "tar";
+						pi.Arguments = "xzf " + target + " --strip-components=1";
 						Process p = Process.Start (pi);
 						p.WaitForExit ();
-							
+						
 						Application.Invoke (delegate {
 							main.Progress = 0;
 							main.Status = "Download complete";
@@ -454,7 +626,14 @@ namespace GuiCompare
 					else {
 						Application.Invoke (delegate {
 							main.Progress = 0;
-							main.Status = "Download failed";
+							FileInfo masterinfoInfo = new FileInfo (masterinfo);
+							if (masterinfoInfo.Exists) {
+								main.Status = "Download failed, reusing cached (possibly out of date) masterinfo";
+								if (done != null)
+									done (masterinfo);
+							}
+							else
+								main.Status = "Download failed";
 						});
 					}
 				}
@@ -462,7 +641,14 @@ namespace GuiCompare
 					Console.WriteLine (e);
 					Application.Invoke (delegate {
 						main.Progress = 0;
-						main.Status = "Download failed";
+						FileInfo masterinfoInfo = new FileInfo (masterinfo);
+						if (masterinfoInfo.Exists) {
+							main.Status = "Download failed, reusing cached (possibly out of date) masterinfo";
+							if (done != null)
+								done (masterinfo);
+						}
+						else
+							main.Status = "Download failed";
 					});
 				}
 			});
@@ -483,11 +669,11 @@ namespace GuiCompare
 		/// <param name="assemblyname">
 		/// The name of the assembly to compare, in this case "mscorlib"
 		/// </param>
-		void StartPresetCompare (string assemblyfile, string profile, string assemblyname)
+		void StartPresetCompare (string assemblyfile, string profile, string assemblyname, string groupName)
 		{
 			Ensure (profile, assemblyname, delegate (string masterinfo){
 				CompareDefinition cd = new CompareDefinition (true, masterinfo, false, assemblyfile);
-				cd.Title = assemblyname;
+				cd.Title = assemblyname + " (" + groupName + ")";
 				main.Config.AddRecent (cd);
 				PopulateRecent ();
 				main.Config.Save ();
@@ -500,9 +686,11 @@ namespace GuiCompare
 					 });
 					
 				main.StartCompare (delegate {
-					main.Title = String.Format ("{0} to {1}", masterinfo, assemblyfile);
+					main.Title = String.Format ("{0} to {1}", assemblyfile, masterinfo);
 				});
 			});
+			
+			main.SetComparedProfile (profile);
 		}
 
 		void Populate (Menu container, string caption, string pdir, string collection, string [] elements)
@@ -511,7 +699,7 @@ namespace GuiCompare
 			string MONO_GAC_PREFIX = Environment.GetEnvironmentVariable ("MONO_GAC_PREFIX");
 			string[] gac_prefixes = null;
 
-			if (MONO_GAC_PREFIX != null)
+			if (!string.IsNullOrEmpty (MONO_GAC_PREFIX))
 				gac_prefixes = MONO_GAC_PREFIX.Split (':');
 
 			MenuItem item = new MenuItem (caption);
@@ -523,7 +711,7 @@ namespace GuiCompare
 				
 				if (e == String.Empty){
 					// Avoid inserting separators twice
-					if (child is SeparatorMenuItem)
+					if (child is SeparatorMenuItem || sub.Children.Length == 0)
 						continue;
 					child = new SeparatorMenuItem ();
 				} else {
@@ -561,14 +749,16 @@ namespace GuiCompare
 					string element = e;
 					child = new MenuItem (e);
 					child.Activated += delegate {
-						StartPresetCompare (assemblyfile, collection, element);
+						StartPresetCompare (assemblyfile, collection, element, caption);
 					};
 				}
 				sub.Add (child);
 			}
-			
-			item.ShowAll ();
-			container.Add (item);
+
+			if (sub.Children.Length > 0) {
+				item.ShowAll ();
+				container.Add (item);
+			}
 		}
 		
 		/// <summary>
@@ -608,12 +798,17 @@ namespace GuiCompare
 		// 
 		// Constructor
 		//
-		public InfoManager (MainWindow main)
+		public InfoManager (MainWindow main, string profilePath)
 		{
 			this.main = main;
 			
-			string corlibdir = System.IO.Path.GetDirectoryName (typeof (int).Assembly.Location);
-			monodir = System.IO.Path.GetFullPath (System.IO.Path.Combine (corlibdir, "..")); 
+			if (profilePath == null) {
+				string corlibdir = System.IO.Path.GetDirectoryName (typeof (int).Assembly.Location);
+				monodir = System.IO.Path.GetFullPath (System.IO.Path.Combine (corlibdir, "..")); 
+			} else {
+				monodir = profilePath;
+			}
+			
 			moondir = System.IO.Path.Combine (monodir, @"../moonlight/plugin");
 	
 			// Work around limitation in Stetic, there is no way
@@ -661,17 +856,19 @@ namespace GuiCompare
 			sub.Add (separator);
 			
 			Populate (sub, "API 1.1", GetVersionPath ("1.0", "net_1_1"), "1.0", api_1_1);
-			Populate (sub, "API 2.0 sp1", GetVersionPath ("2.0", "net_2_0"), "2.0", api_2_0);
-			Populate (sub, "API 3.0 sp1", GetVersionPath ("3.0", "net_3_0"), "3.0", api_3_0);
+			Populate (sub, "API 2.0 sp2", GetVersionPath ("2.0", "net_2_0"), "2.0", api_2_0);
+			Populate (sub, "API 3.0 sp1", GetVersionPath ("2.0", "net_2_0"), "3.0", api_3_0);
 			Populate (sub, "API 3.5 sp1", GetVersionPath ("2.0", "net_2_0"), "3.5", api_3_5);
-			Populate (sub, "API 4.0 beta 1", GetVersionPath ("4.0", "net_4_0"), "4.0", api_4_0);
-			Populate (sub, "Silverlight 2.0", GetVersionPath ("2.1", "net_2_1"), "SL2", api_sl2);
-			Populate (sub, "Silverlight 3.0", GetVersionPath ("2.1", "net_2_1"), "SL3", api_sl2);
+			Populate (sub, "API 4.0", GetVersionPath ("4.0", "net_4_0"), "4.0", api_4_0);
+			Populate (sub, "API 4.5", GetVersionPath ("4.5", "net_4_5"), "4.5", api_4_5);
+//			Populate (sub, "Silverlight 2.0", GetVersionPath ("2.1", "net_2_1"), "SL2", api_sl2);
+//			Populate (sub, "Silverlight 3.0", GetVersionPath ("2.1", "net_2_1"), "SL3", api_sl2);
+			Populate (sub, "Silverlight 4.0", GetVersionPath ("2.1", "net_2_1"), "SL4", api_sl4);
 		}
 		
-		static string GetVersionPath (string version, string profile)
+		string GetVersionPath (string version, string profile)
 		{
-			if (string.IsNullOrEmpty (Environment.GetEnvironmentVariable ("MONO_PATH")))
+			if (!monodir.Contains (Path.Combine ("mcs", "class", "lib")))
 				return version;
 
 			// Developer's version pointing to /mcs/class/lib/<profile>/
