@@ -205,8 +205,13 @@ namespace GuiCompare {
 				string name = n.Attributes ["name"].Value;
 				if (CheckIfAdd (name, n)) {
 					string key = GetNodeKey (name, n);
-					//keys.Add (key, name);
-					keys [key] = name;
+					if (keys.Contains (key)) {
+						if ((string) keys[key] != name)
+							throw new NotImplementedException ("Attribute with same name but diffent value");
+					} else {
+						keys.Add (key, name);
+					}
+					
 					LoadExtraData (key, n);
 				}
 			}
@@ -532,32 +537,24 @@ namespace GuiCompare {
 
 	public class XMLAttributeProperties: XMLNameGroup
 	{
-		static Hashtable ignored_properties;
+		SortedDictionary <string, string> properties;
+
 		static XMLAttributeProperties ()
 		{
-			ignored_properties = new Hashtable ();
-			ignored_properties.Add ("System.Reflection.AssemblyKeyFileAttribute", "KeyFile");
-			ignored_properties.Add ("System.Reflection.AssemblyCompanyAttribute", "Company");
-			ignored_properties.Add ("System.Reflection.AssemblyConfigurationAttribute", "Configuration");
-			ignored_properties.Add ("System.Reflection.AssemblyCopyrightAttribute", "Copyright");
-			ignored_properties.Add ("System.Reflection.AssemblyProductAttribute", "Product");
-			ignored_properties.Add ("System.Reflection.AssemblyTrademarkAttribute", "Trademark");
-			ignored_properties.Add ("System.Reflection.AssemblyInformationalVersionAttribute", "InformationalVersion");
-
-			ignored_properties.Add ("System.ObsoleteAttribute", "Message");
-			ignored_properties.Add ("System.IO.IODescriptionAttribute", "Description");
-			ignored_properties.Add ("System.Diagnostics.MonitoringDescriptionAttribute", "Description");
 		}
 
-		Hashtable properties = new Hashtable ();
 		string attribute;
 
+		public XMLAttributeProperties ()
+			: this (null)
+		{}
+
 		public XMLAttributeProperties (string attribute)
 		{
 			this.attribute = attribute;
 		}
 
-		public override void LoadData(XmlNode node)
+		public override void LoadData (XmlNode node)
 		{
 			if (node == null)
 				throw new ArgumentNullException ("node");
@@ -565,19 +562,23 @@ namespace GuiCompare {
 			if (node.ChildNodes == null)
 				return;
 
-			string ignored = ignored_properties [attribute] as string;
 
 			foreach (XmlNode n in node.ChildNodes) {
-				string name = n.Attributes ["name"].Value;
-				if (ignored == name)
-					continue;
+				string name = n.Attributes["name"].Value;
 
-				if (n.Attributes ["null"] != null) {
-					properties.Add (name, null);
+				if (n.Attributes["null"] != null) {
+					Properties.Add (name, null);
 					continue;
 				}
-				string value = n.Attributes ["value"].Value;
-				properties.Add (name, value);
+				Properties.Add (name, n.Attributes ["value"].Value);
+			}
+		}
+
+		public IDictionary <string, string> Properties {
+			get {
+				if (properties == null)
+					properties = new SortedDictionary <string, string> ();
+				return properties;
 			}
 		}
 
@@ -596,10 +597,9 @@ namespace GuiCompare {
 
 	public class XMLAttributes : XMLNameGroup
 	{
-		Hashtable properties = new Hashtable ();
-
 		bool isTodo;
 		string comment;
+		SortedDictionary <string, XMLAttributeProperties> properties;
 
 		protected override bool CheckIfAdd (string value, XmlNode node)
 		{
@@ -662,7 +662,7 @@ namespace GuiCompare {
 			return key;
 		}
 
-		protected override void LoadExtraData(string name, XmlNode node)
+		protected override void LoadExtraData (string name, XmlNode node)
 		{
 			XmlNode pNode = node.SelectSingleNode ("properties");
 
@@ -676,15 +676,27 @@ namespace GuiCompare {
 			
 			if (MasterUtils.IsImplementationSpecificAttribute (name))
 				return;
-			
+
 			if (pNode != null) {
 				XMLAttributeProperties p = new XMLAttributeProperties (name);
 				p.LoadData (pNode);
 
-				properties[name] = p;
+				IDictionary <string, XMLAttributeProperties> properties = Properties;
+				if (properties.ContainsKey (name))
+					properties [name] = p;
+				else
+					properties.Add (name, p);
 			}
 		}
-		
+
+		public IDictionary <string, XMLAttributeProperties> Properties {
+			get {
+				if (properties == null)
+					properties = new SortedDictionary <string, XMLAttributeProperties> ();
+				return properties;
+			}
+		}
+
 		public override string GroupName {
 			get { return "attributes"; }
 		}
@@ -1018,6 +1030,11 @@ namespace GuiCompare {
 
 		public override string GetNodeKey (string name, XmlNode node)
 		{
+			XmlNode genericNode = node.SelectSingleNode ("generic-parameters");
+			if (genericNode != null) {
+				name = name + "`" + genericNode.ChildNodes.Count;
+			}
+			
 			// for explicit/implicit operators we need to include the return
 			// type in the key to allow matching; as a side-effect, differences
 			// in return types will be reported as extra/missing methods
@@ -1076,7 +1093,7 @@ namespace GuiCompare {
 			get { return "constructor"; }
 		}
 	}
-
+/*
 	public class XmlNodeComparer : IComparer
 	{
 		public static XmlNodeComparer Default = new XmlNodeComparer ();
@@ -1088,4 +1105,5 @@ namespace GuiCompare {
 			return String.Compare (na.Attributes ["name"].Value, nb.Attributes ["name"].Value);
 		}
 	}
+*/ 
 }
